"""
Django settings for Meditation project.

Generated by 'django-admin startproject' using Django 2.0.

For more information on this file, see
https://docs.djangoproject.com/en/2.0/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/2.0/ref/settings/
"""

import os
from os import path

from dotenv import load_dotenv

load_dotenv()

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/3.0/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = os.environ.get("DJANGO_SECRET_KEY")

# SECURITY WARNING: don't run with debug turned on in production!
#DEBUG = True if os.environ.get("DEBUG", "true") == "true" else False
DEBUG = True

ALLOWED_HOSTS = ["*"]

SITE_ID = 1
# Application definition


INSTALLED_APPS = [
    "django.contrib.admin",
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "rest_framework",
    "rest_framework.authtoken",
    "corsheaders",
    "rest_framework_swagger",
    "django_filters",
    "django.contrib.sites",
    "rest_auth",
    "rest_auth.registration",
    "allauth",
    "MeditationApp",
    "fcm_django",
    "sslserver",
    "djrichtextfield",
    "background_task",
    "drf_yasg",
    "django_extensions"
]

MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "corsheaders.middleware.CorsMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
    "Meditation.middleware.RangesMiddleware",
]

AUTH_USER_MODEL = "MeditationApp.User"
ROOT_URLCONF = "Meditation.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [os.path.join(BASE_DIR, "templates/")],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.template.context_processors.debug",
                "django.template.context_processors.request",
                "django.contrib.auth.context_processors.auth",
                "django.contrib.messages.context_processors.messages",
            ],
            "libraries": {
                "staticfiles": "django.templatetags.static",
            },
        },
    },
]

WSGI_APPLICATION = 'Meditation.wsgi.application'


# Database
# https://docs.djangoproject.com/en/3.0/ref/settings/#databases

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": os.path.join(BASE_DIR, "db.sqlite3"),
    }
}

# Password validation
# https://docs.djangoproject.com/en/3.0/ref/settings/#auth-password-validators

AUTH_PASSWORD_VALIDATORS = [
    {
        "NAME": "django.contrib.auth.password_validation.UserAttributeSimilarityValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.MinimumLengthValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.CommonPasswordValidator",
    },
    {
        "NAME": "django.contrib.auth.password_validation.NumericPasswordValidator",
    },
]

FCM_DJANGO_SETTINGS = {
    "FCM_SERVER_KEY": os.environ.get("FCM_SERVER_KEY")
}
# Internationalization
# https://docs.djangoproject.com/en/3.0/topics/i18n/
BACKGROUND_TASK_ASYNC_THREADS = 300
AUTHENTICATION_BACKENDS = [
    "django.contrib.auth.backends.ModelBackend",
    "MeditationApp.backends.CustomerBackend",
]
SWAGGER_SETTINGS = {
    "SECURITY_DEFINITIONS": {"basic": {"type": "basic"}},
}

DJRICHTEXTFIELD_CONFIG = {
    "js": ["//tinymce.cachefly.net/4.1/tinymce.min.js"],
    "init_template": "djrichtextfield/init/tinymce.js",
    "settings": {
        "menubar": True,
        "plugins": "link image",
        "toolbar": "bold italic | link image | removeformat",
        "width": 700,
    },
}

LOGIN_URL = "rest_framework:login"
LOGOUT_URL = "rest_framework:logout"

LANGUAGE_CODE = "en-us"

TIME_ZONE = "Asia/Kolkata"

USE_I18N = True

USE_L10N = True

USE_TZ = False

REST_FRAMEWORK = {
    "DEFAULT_AUTHENTICATION_CLASSES": (
        "MeditationApp.backends.JWTAuthentication",
        "rest_framework.authentication.SessionAuthentication",
    ),
    "DEFAULT_PERMISSION_CLASSES": (
        "rest_framework.permissions.IsAuthenticated",
        "rest_framework.permissions.AllowAny",
        # "rest_framework.permissions.DjangoModelPermissions",
    ),
    "DEFAULT_PARSER_CLASSES": (
        "rest_framework.parsers.JSONParser",
        "rest_framework.parsers.FormParser",
        "rest_framework.parsers.MultiPartParser",
    ),
    "DEFAULT_FILTER_BACKENDS": ("django_filters.rest_framework.DjangoFilterBackend",),
    "DEFAULT_PAGINATION_CLASS": "rest_framework.pagination.PageNumberPagination",
    "EXCEPTION_HANDLER": "MeditationApp.exceptions.core_exception_handler",
    "NON_FIELD_ERRORS_KEY": "error",
    'DEFAULT_SCHEMA_CLASS': 'rest_framework.schemas.coreapi.AutoSchema' 
}

CORS_ORIGIN_ALLOW_ALL = True
# CORS_ALLOW_CREDENTIALS = True

CORS_ALLOW_METHODS = [
    "DELETE",
    "GET",
    "OPTIONS",
    "PATCH",
    "POST",
    "PUT",
]
APPEND_SLASH = True
CORS_ALLOW_HEADERS = [
    "accept",
    "accept-encoding",
    "authorization",
    "content-type",
    "dnt",
    "origin",
    "crm_manager-agent",
    "x-csrftoken",
    "x-requested-with",
]

DJRICHTEXTFIELD_CONFIG = {
    "js": ["//tinymce.cachefly.net/4.1/tinymce.min.js"],
    "init_template": "djrichtextfield/init/tinymce.js",
    "settings": {
        "menubar": True,
        "plugins": "link image",
        "toolbar": "bold italic | link image | removeformat",
        "width": 700,
    },
}

ACCOUNT_LOGOUT_ON_GET = True

# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/3.0/howto/static-files/

MEDIA_URL = "/media/"
MEDIA_ROOT = os.path.join(BASE_DIR, "media")
STATIC_ROOT = path.join(BASE_DIR, "static")
STATIC_URL = "/static/"

# Send Email Configuration
EMAIL_BACKEND = "django.core.mail.backends.smtp.EmailBackend"
EMAIL_HOST = "smtp.gmail.com"
EMAIL_USE_TLS = True
EMAIL_PORT = 587
EMAIL_HOST_USER = os.environ.get("EMAIL_HOST_USER")
EMAIL_HOST_PASSWORD = os.environ.get("EMAIL_HOST_PASSWORD")

MESSAGE_STORAGE = "django.contrib.messages.storage.session.SessionStorage"

if DEBUG == False:
    ALLOWED_HOSTS = ["*"]

    CSRF_COOKIE_SECURE = True
    CSRF_COOKIE_HTTPONLY = True

    SECURE_HSTS_SECONDS = 60 * 60 * 24 * 7 * 52  # one year
    SECURE_HSTS_INCLUDE_SUBDOMAINS = True
    SECURE_SSL_REDIRECT = True
    SECURE_BROWSER_XSS_FILTER = True
    SECURE_CONTENT_TYPE_NOSNIFF = True
    SECURE_PROXY_SSL_HEADER = ("HTTP_X_FORWARDED_PROTO", "https")

    SESSION_COOKIE_SECURE = True
    SECURE_HSTS_PRELOAD = True
    SECURE_REFERRER_POLICY = "same-origin"

    AWS_ACCESS_KEY_ID = os.environ.get("S3_KEY")
    AWS_SECRET_ACCESS_KEY = os.environ.get("S3_SECRET")
    AWS_STORAGE_BUCKET_NAME = os.environ.get("S3_BUCKET")
    AWS_S3_CUSTOM_DOMAIN = "%s.s3.amazonaws.com" % AWS_STORAGE_BUCKET_NAME

    AWS_S3_OBJECT_PARAMETERS = {
        "CacheControl": "max-age=86400",
    }
    AWS_LOCATION = 'media'
    STATIC_URL = "https://%s/%s/" % (AWS_S3_CUSTOM_DOMAIN, AWS_LOCATION)
    DEFAULT_FILE_STORAGE = 'storages.backends.s3boto3.S3Boto3Storage'
    STATICFILES_STORAGE = 'storages.backends.s3boto3.S3StaticStorage'
    APPEND_SLASH=True

    MIDDLEWARE += ["csp.middleware.CSPMiddleware"]
    CSP_DEFAULT_SRC = ("'self'", 'meditationapp-srv.s3.amazonaws.com')
    CSP_SCRIPT_SRC = ("'self'", 'meditationapp-srv.s3.amazonaws.com')
    CSP_IMG_SRC = ("'self'", 'meditationapp-srv.s3.amazonaws.com')
    CSP_OBJECT_SRC = ("'self'", 'meditationapp-srv.s3.amazonaws.com')
    CSP_BASE_URI = ("'self'", 'meditationapp-srv.s3.amazonaws.com')
    CSP_FRAME_ANCESTORS = ("'self'", 'meditationapp-srv.s3.amazonaws.com')
    CSP_FORM_ACTION = ("'self'", 'meditationapp-srv.s3.amazonaws.com')
    CSP_INCLUDE_NONCE_IN = ('script-src', 'meditationapp-srv.s3.amazonaws.com')
    CSP_MANIFEST_SRC = ("'self'", 'meditationapp-srv.s3.amazonaws.com')
    CSP_WORKER_SRC = ("'self'",'meditationapp-srv.s3.amazonaws.com')
    CSP_MEDIA_SRC = ("'self'", 'meditationapp-srv.s3.amazonaws.com')
    CSP_FONT_SRC = ("'self'", 'meditationapp-srv.s3.amazonaws.com')
    CSP_STYLE_SRC = ("'self'", 'meditationapp-srv.s3.amazonaws.com')

LOGGING = {
    "version": 1,
    "disable_existing_loggers": True,
    "formatters": {
        "console": {"format": "%(name)-12s %(levelname)-8s %(message)s"},
        "file": {
            "format": "%(created)f | %(asctime)s | %(name)-12s | %(levelname)s | %(message)s"
        },
    },
    "handlers": {
        "file": {
            "level": "DEBUG",
            "class": "logging.FileHandler",
            "filename": BASE_DIR + "/debug.log",
            "formatter": "file",
        },
        "console": {
            "class": "logging.StreamHandler",
        },
    },
    "loggers": {
        "django.server": {
            "handlers": ["file", "console"],
            "level": "DEBUG",
            "propagate": True,
            "level": "DEBUG",
        },
        "django.request": {
            "handlers": ["file", "console"],
            "level": "DEBUG",
            "propagate": True,
            "level": "DEBUG",
        },
    },
}

DEFAULT_AUTO_FIELD='django.db.models.BigAutoField'