"""Meditation URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/2.2/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""

from django.conf import settings
from django.conf.urls import url
from django.conf.urls.static import static
from django.contrib import admin
from django.urls import include, path
from drf_yasg import openapi
from drf_yasg.views import get_schema_view
from rest_framework import permissions
from rest_framework_swagger.views import get_swagger_view

schema_view = get_swagger_view(title=" MeditationAPI")
schema_view_redoc = get_schema_view(
    openapi.Info(
        title="MeditationApp API",
        default_version="v1",
        description="",
        contact=openapi.Contact(email="contact@snippets.local"),
        license=openapi.License(name="BSD License"),
    ),
    public=True,
    permission_classes=[permissions.AllowAny],
)

urlpatterns = [
    path("api/v1/apilist/", schema_view),
    path("admin/", admin.site.urls),
    path("api/v1/", include("MeditationApp.urls")),
    url(r"^api-auth/", include("rest_framework.urls", namespace="rest_framework")),
    path("djrichtextfield/", include("djrichtextfield.urls")),
    url(
        r"^redoc/$",
        schema_view_redoc.with_ui("redoc", cache_timeout=0),
        name="schema-redoc",
    ),
]
if settings.DEBUG:
    urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)
    urlpatterns += static(settings.STATIC_URL,
                          document_root=settings.STATIC_ROOT)

admin.site.site_header = "Soul Admin"
admin.site.site_title = "Soul"
admin.site.index_title = "Soul"
