from django.contrib import admin
from django.contrib.auth.models import Group
from django.contrib.sites.models import Site
from rest_framework.authtoken.models import Token

from MeditationApp.models import DownloadedSongs

from .models import (OTP, CategoryImage, Contact, EffectAlbum, EffectCategory,
                     EffectSongs, ExploreSongs, ExploreVideos, LoginPageVideo,
                     Page, Personlize, Songs, SongsCategory, SongsImage,
                     TodaySpecial, TodaySpecialFile, User, UserFavourite,
                     UserNotification, UserSubscription)

admin.autodiscover()

# Register your Models Here

# Unregister your Models Here
admin.site.unregister(Site)
admin.site.unregister(Group)
admin.site.unregister(Token)


class UserAdmin(admin.ModelAdmin):
    exclude = (
        "user_permissions",
        "groups",
        "first_name",
        "last_name",
        "password",
        "is_staff",
        "is_superuser",
        "date_joined",
    )

class BaseAdmin(admin.ModelAdmin):
    list_display = ('id', 'name', 'status',)
    list_filter = ('status',)
    search_fields = ('name','title',)
    exclude = ["isDeleted", "title"]    

    def save_model(self, request, obj, form, change):
        """
        Given a model instance save it to the database.
        """
        obj.title = form.data['name']
        obj.save()


class EffectSongsAdmin(admin.ModelAdmin):
    list_display = ('id', 'name', 'status',)
    list_filter = ('status',)
    search_fields = ('name','title',)
    # readonly_fields = ('listen_count',)
    exclude = ["isDeleted", "title"]    

    def save_model(self, request, obj, form, change):
        """
        Given a model instance save it to the database.
        """
        obj.title = form.data['name']
        obj.save()


class TodaySpecialFileAdmin(admin.ModelAdmin):
    list_display = ('id', 'name',)
    search_fields = ('name','title',)
    readonly_fields = ('listen_count',)
    exclude = ["isDeleted", "title"]    

    def save_model(self, request, obj, form, change):
        """
        Given a model instance save it to the database.
        """
        obj.title = form.data['name']
        obj.save()


class ExploreSongsAdmin(admin.ModelAdmin):
    list_display = ('id','created','updated')
    search_fields = ('id','songs__title',)
    exclude = ["isDeleted",]    


class TodaySpecialAdmin(admin.ModelAdmin):
    list_display = ('id','status','created','updated')
    search_fields = ('id','video__title',)
    exclude = ["isDeleted",]    


class ExploreVideosAdmin(admin.ModelAdmin):
    list_display = ('id','name',)
    search_fields = ('id','name',)


class LoginPageVideoAdmin(admin.ModelAdmin):
    list_display = ('id','name','status')
    search_fields = ('id','name','status')
    date_hierarchy = 'created'
    exclude = ["isDeleted",]    


class PersonlizeAdmin(admin.ModelAdmin):
    list_display = ('id','user','created', 'updated')
    search_fields = ('id','user__username',)
    date_hierarchy = 'created'
    exclude = ["isDeleted",]    

class UserFavouriteAdmin(admin.ModelAdmin):
    list_display = ('id','user','songs','status','created', 'updated')
    search_fields = ('id','user__username',)
    date_hierarchy = 'created'
    exclude = ["isDeleted",]    
    readonly_fields = ('listen_count',)


class PageAdmin(admin.ModelAdmin):
    list_display = ('id','title','pagerole','created', 'updated')
    search_fields = ('id','title','pagerole')
    date_hierarchy = 'created'


class ContactAdmin(admin.ModelAdmin):
    list_display = ('id','email','phone', 'created', 'updated')
    list_display_links = ('id',)
    date_hierarchy = 'created'
    search_fields = ('first_name','last_name','email','phone')

class CategoryImageAdmin(admin.ModelAdmin):
    list_display = ['id', 'get_image_url',]
    list_display_links = ['id', 'get_image_url',]

class DownloadedSongsAdmin(admin.ModelAdmin):
    list_display = ['id', 'user','device_type', 'created', 'updated']
    search_fields = ('user__first_name','user__last_name','user__email','user__phone', 'device_type')
    date_hierarchy = 'created'

admin.site.register(CategoryImage, CategoryImageAdmin)
admin.site.register(Contact,ContactAdmin)
admin.site.register(DownloadedSongs, DownloadedSongsAdmin)
admin.site.register(EffectAlbum, BaseAdmin)
admin.site.register(SongsCategory, BaseAdmin)
admin.site.register(SongsImage, CategoryImageAdmin)
admin.site.register(Songs, BaseAdmin)
admin.site.register(Personlize, PersonlizeAdmin)
admin.site.register(TodaySpecialFile, TodaySpecialFileAdmin)
admin.site.register(TodaySpecial, TodaySpecialAdmin)
admin.site.register(UserFavourite, UserFavouriteAdmin)
admin.site.register(EffectCategory, BaseAdmin)
admin.site.register(EffectSongs, EffectSongsAdmin)
admin.site.register(ExploreSongs, ExploreSongsAdmin)
admin.site.register(ExploreVideos, ExploreVideosAdmin)
admin.site.register(Page,PageAdmin)
admin.site.register(LoginPageVideo, LoginPageVideoAdmin)
admin.site.register(UserNotification)
admin.site.register(UserSubscription)
admin.site.register(User, UserAdmin)
