from datetime import datetime, timedelta
import cv2
from django.utils.timezone import get_current_timezone
from mutagen.mp3 import MP3
from rest_framework import serializers
from django.conf import settings
from MeditationApp.models import (CategoryImage, Contact, DownloadedSongs, EffectAlbum,
                                  EffectCategory, EffectSongs, ExploreSongs,
                                  ExploreVideos, FavouriteVideos,
                                  LoginPageVideo, Page, Personlize,
                                  Recommendation, Songs, SongsCategory,
                                  SongsImage, TodaySpecial, TodaySpecialFile,
                                  User, UserFavourite, UserNotification,
                                  UserSubscription, UserVideoFavourite)
from MeditationApp.utils import validate_subcription


class UserSerializerList(serializers.HyperlinkedModelSerializer):
    class Meta:
        model = User
        fields = (
            "id",
            "username",
            "gender",
            "firstname",
            "lastname",
            "email",
            "image",
            "phone",
            "work",
        )


class UserSerializer(serializers.ModelSerializer):
    class Meta:
        model = User
        fields = (
            "id",
            "username",
            "email",
            "firstname",
            "gender",
            "lastname",
            "password",
        )
        extra_kwargs = {
            "password": {"error_messages": {"required": "Please enter password"}}
        }


class UserUpdateSerializer(serializers.ModelSerializer):
    class Meta:
        model = User
        fields = (
            "id",
            "username",
            "email",
            "gender",
            "firstname",
            "lastname",
            "image",
            "phone",
            "work",
        )
        extra_kwargs = {
            "email": {"required": False},
            "image": {"required": False},
            "username": {"required": False},
        }


class LoginSerializer(serializers.Serializer):
    email = serializers.CharField(required=True)
    password = serializers.CharField(required=True)
    device_id = serializers.CharField(required=False, allow_blank=True)
    device_token = serializers.CharField(required=False, allow_blank=True)
    device_type = serializers.CharField(required=False, allow_blank=True)


class ChangePasswordSerializer(serializers.Serializer):
    """
    Serializer for password change endpoint.
    """

    old_password = serializers.CharField(required=True)
    new_password = serializers.CharField(required=True)

    # def validate_old_password(self, value):
    #     if not self.context['request'].user.check_password(value):
    #         raise serializers.ValidationError('Current password does not match')
    #     return value


class ForgotPasswordSerializer(serializers.Serializer):
    email = serializers.CharField(required=True)


class ResetPasswordSerializer(serializers.Serializer):
    """
    Serializer for password change endpoint.
    """

    email = serializers.CharField(required=True)
    password = serializers.CharField(required=True)


class ConfirmTokenSerializer(serializers.Serializer):
    """
    Serializer for OTP endpoint.
    """

    email = serializers.CharField(required=True)
    token = serializers.CharField(required=True)


class RecommendationSerializer(serializers.ModelSerializer):
    class Meta:
        model = Recommendation
        fields = ("id", "name")


class PersonlizeSerializer(serializers.ModelSerializer):
    recommendation = serializers.SerializerMethodField()

    class Meta:
        model = Personlize
        fields = ("id", "user", "recommendation")

    def get_recommendation(self, obj):
        try:
            recommendation = obj.recommendation.values_list("name", flat=True)
        except Exception as e:
            recommendation = None
        return recommendation


class SongsImageSerializerList(serializers.ModelSerializer):
    class Meta:
        model = SongsImage
        fields = ["image"]


class SongsSerializerList(serializers.ModelSerializer):
    songs_length = serializers.SerializerMethodField()
    is_user_fav = serializers.SerializerMethodField()
    songs_length_in_sec = serializers.SerializerMethodField()
    author_name = serializers.SerializerMethodField()
    author_image = serializers.SerializerMethodField()

    class Meta:
        model = Songs
        fields = [
            "id",
            "name",
            "songs_length",
            "songs_length_in_sec",
            "is_user_fav",
            "title",
            "audios",
            "cover_images",
            "description",
            "listen_count",
            "status",
            "author_name",
            "author_image",
        ]

    def get_author_name(self, obj):
        try:
            author_name = obj.songs_category.author_name
        except Exception as e:
            author_name = None
        return author_name

    def get_author_image(self, obj):
        try:
            author_image = obj.songs_category.author_image
            request = self.context.get("request")
            author_image = request.build_absolute_uri(author_image.url)
        except Exception as e:
            author_image = None
        return str(author_image)

    def get_is_user_fav(self, obj):
        try:
            user = self.context.get("request").user
            song_id = obj.id
            userFavourite = UserFavourite.objects.filter(
                user=user, songs=song_id, isDeleted=False
            ).first()
        except Exception as e:
            userFavourite = None
        status = True if userFavourite else False
        return status

    def get_songs_length(self, obj):
        audios = obj.audios
        try:
            audio = MP3(audios)
            length = audio.info.length
            min, sec = divmod(length, 60)
            hour, min = divmod(min, 60)
            # "%d:%02d:%02d" % (hour, min, sec)
            length = "%02d:%02d" % (min, sec)
        except:
            length = None
        return length

    def get_songs_length_in_sec(self, obj):
        audios = obj.audios
        try:
            audio = MP3(audios)
            length = audio.info.length
        except:
            length = None
        return length


class CategoryImageSerializerList(serializers.ModelSerializer):
    class Meta:
        model = CategoryImage
        fields = ["image"]


class SongsCategorySerializerList(serializers.ModelSerializer):
    songs = serializers.SerializerMethodField()

    class Meta:
        model = SongsCategory
        fields = [
            "id",
            "name",
            "title",
            "author_name",
            "author_image",
            "cover_images",
            "status",
            "description",
            "songs",
        ]

    def get_songs(self, obj):
        serializer_context = {"request": self.context.get("request")}
        return SongsSerializerList(
            obj.category_songs.all(), many=True, context=serializer_context
        ).data


class TodaySpecialFileSerializerList(serializers.ModelSerializer):
    file = serializers.SerializerMethodField()
    def get_file(self,obj):
        if obj.file:
            file_url = f'{"https://api.souljourney.io"}/{obj.file.url}'
        else:
            file_url = ''
        return file_url
    
    class Meta:
        model = TodaySpecialFile
        fields = ["id", "file"]


class TodaySpecialSerializerList(serializers.ModelSerializer):
    video = TodaySpecialFileSerializerList()
    name = serializers.SerializerMethodField()
    title = serializers.SerializerMethodField()
    listen_count = serializers.SerializerMethodField()

    class Meta:
        model = TodaySpecial
        fields = ["id", "name", "title", "video", "listen_count", "status"]

    def get_name(self, obj):
        try:
            name = obj.video.name
        except:
            name = None
        return name

    def get_title(self, obj):
        try:
            title = obj.video.title
        except:
            title = None
        return title

    def get_listen_count(self, obj):
        try:
            listen_count = obj.video.listen_count
        except:
            listen_count = None
        return listen_count


class UserFavouriteSerializer(serializers.ModelSerializer):
    class Meta:
        model = UserFavourite
        fields = ["id", "user", "songs", "listen_count", "status"]


class UserVideoFavouriteSerializer(serializers.ModelSerializer):
    class Meta:
        model = UserVideoFavourite
        fields = ["id", "user", "videos",]


class SocialAuthSerializer(serializers.Serializer):
    token = serializers.CharField(required=True)
    device_id = serializers.CharField(required=False, allow_blank=True)
    device_token = serializers.CharField(required=False, allow_blank=True)
    device_type = serializers.CharField(required=False, allow_blank=True)


class SocialAuthAppleSerializer(serializers.Serializer):
    name = serializers.CharField(required=True)
    username = serializers.CharField(required=True)
    email = serializers.CharField(required=True)
    device_id = serializers.CharField(required=False, allow_blank=True)
    device_token = serializers.CharField(required=False, allow_blank=True)
    device_type = serializers.CharField(required=False, allow_blank=True)


class UpdateTodayspecialVideoSerializer(serializers.Serializer):
    id = serializers.CharField(required=True)


class SocialAuthGoogleSerializer(serializers.Serializer):
    name = serializers.CharField(required=True)
    username = serializers.CharField(required=True)
    email = serializers.CharField(required=True)


class EffectSongsSerializerList(serializers.ModelSerializer):
    songs_length = serializers.SerializerMethodField()
    songs_length_in_sec = serializers.SerializerMethodField()
    author_name = serializers.SerializerMethodField()
    author_image = serializers.SerializerMethodField()

    class Meta:
        model = EffectSongs
        fields = [
            "id",
            "name",
            "songs_length",
            "songs_length_in_sec",
            "title",
            "audios",
            "cover_images",
            "description",
            "status",
            "author_name",
            "author_image",
        ]

    def get_author_name(self, obj):
        try:
            author_name = obj.effect_album.effect_category.author_name
        except Exception as e:
            author_name = None
        return author_name

    def get_author_image(self, obj):
        try:
            author_image = obj.effect_album.effect_category.author_image
            request = self.context.get("request")
            author_image = request.build_absolute_uri(author_image.url)
        except Exception as e:
            author_image = None
        return str(author_image)

    def get_songs_length(self, obj):
        audios = obj.audios
        try:
            audio = MP3(audios)
            length = audio.info.length
            min, sec = divmod(length, 60)
            hour, min = divmod(min, 60)
            # "%d:%02d:%02d" % (hour, min, sec)
            length = "%02d:%02d" % (min, sec)
        except:
            length = None
        return length

    def get_songs_length_in_sec(self, obj):
        audios = obj.audios
        try:
            audio = MP3(audios)
            length = audio.info.length
        except:
            length = None
        return length


class EffectAlbumSerializerList(serializers.ModelSerializer):
    effect_songs = serializers.SerializerMethodField()

    class Meta:
        model = EffectAlbum
        fields = [
            "id",
            "name",
            "title",
            "cover_images",
            "description",
            "status",
            "effect_songs",
        ]

    def get_effect_songs(self, obj):
        serializer_context = {"request": self.context.get("request")}
        return EffectSongsSerializerList(
            obj.effect_albums_songs.all(), many=True, context=serializer_context
        ).data


class EffectCategorySerializerList(serializers.ModelSerializer):
    effect_album = serializers.SerializerMethodField()

    class Meta:
        model = EffectCategory
        fields = [
            "id",
            "name",
            "title",
            "author_name",
            "author_image",
            "cover_images",
            "status",
            "description",
            "effect_album",
        ]

    def get_effect_album(self, obj):
        serializer_context = {"request": self.context.get("request")}
        return EffectAlbumSerializerList(
            obj.category_effects_albums.all(), many=True, context=serializer_context
        ).data


class ExploreVideosList(serializers.ModelSerializer):
    duration = serializers.SerializerMethodField()
    is_user_fav = serializers.SerializerMethodField()

    class Meta:
        model = ExploreVideos
        fields = ["id", "name", "file", "duration", "is_user_fav"]

    def get_duration(self, obj):
        try:
            url = obj.file.url
            data = cv2.VideoCapture(url)
            
            # count the number of frames
            frames = data.get(cv2.CAP_PROP_FRAME_COUNT)
            fps = data.get(cv2.CAP_PROP_FPS)
            
            # calculate duration of the video
            seconds = round(frames / fps)
            duration = timedelta(seconds=seconds)
        except Exception as e:
            print(">>>>>>>>..", e)
            duration = timedelta(seconds=0)
        return str(duration)

    def get_is_user_fav(self, obj):
        try:
            user = self.context.get("request").user
            videos_id = obj.id
            userFavourite = FavouriteVideos.objects.filter(
                user=user, videos=videos_id, isDeleted=False
            ).first()
        except Exception as e:
            userFavourite = None
        status = True if userFavourite else False
        return status


class ExploreSongsList(serializers.ModelSerializer):
    videos = serializers.SerializerMethodField()
    songs = serializers.SerializerMethodField()

    class Meta:
        model = ExploreSongs
        fields = ["id", "songs", "status", "videos"]

    def get_songs(self, obj):
        serializer_context = {"request": self.context.get("request")}
        return SongsSerializerList(
            obj[0].songs.all(), many=True, context=serializer_context
        ).data

    def get_videos(self, obj):
        serializer_context = {"request": self.context.get("request")}
        return ExploreVideosList(
            obj[0].videos.all(), many=True, context=serializer_context
        ).data


class PageSerializerList(serializers.ModelSerializer):
    class Meta:
        model = Page
        fields = ["id", "title", "pagerole", "description"]


class ContactUsSerializer(serializers.ModelSerializer):
    class Meta:
        model = Contact
        fields = ["id", "first_name", "email", "message"]


class LoginPageVideoSerializerList(serializers.ModelSerializer):
    class Meta:
        model = LoginPageVideo
        fields = ["id", "video", "name", "status"]


class AddAlarmNotificationSerializer(serializers.Serializer):
    user = serializers.CharField(required=True)
    category_id = serializers.CharField(required=True)
    set_time = serializers.CharField(required=True)
    end_time = serializers.CharField(required=True)


class AddDurationNotificationSerializer(serializers.Serializer):
    user = serializers.CharField(required=True)
    category_id = serializers.CharField(required=True)
    current_time = serializers.CharField(required=True)
    duration_in_min = serializers.CharField(required=True)


class UserNotificationSerializerList(serializers.ModelSerializer):
    time = serializers.SerializerMethodField()

    class Meta:
        model = UserNotification
        fields = ["id", "title", "message", "image", "created", "time"]

    def get_time(self, obj):
        created_time = obj.created
        current_time = datetime.now()
        time = current_time - created_time
        return str(time.microseconds)


class SubscriptionReceiptSerializer(serializers.Serializer):
    user = serializers.CharField(required=False)
    receipt = serializers.CharField(required=True)
    productid = serializers.CharField(required=True)
    packageName = serializers.CharField(required=True)
    platform = serializers.ChoiceField(
                        choices = (("apple","apple"), ("google","google"),))

    def validate(self, obj):
        subscription = validate_subcription(obj)
        if subscription["success"] == False:
            raise serializers.ValidationError(subscription["message"])
        return super(SubscriptionReceiptSerializer, self).validate(obj)

    def create(self, obj):
        subscription = validate_subcription(obj)

        user_subcription, created = UserSubscription.objects.update_or_create(
            user=User.objects.get(id=obj["user"]),
            defaults={
                "user": User.objects.get(id=obj["user"]),
                "productid": obj["productid"],
                "packageName": obj["packageName"],
                "receipt": obj["receipt"],
                "platform": obj["platform"],
                "status": subscription["success"],
                "autorenewing": subscription.get("data").get("receipt").get("autoRenewing", False),
                "price": subscription.get("data").get("receipt").get("priceAmountMicros"),
                "country_code": subscription.get("data").get("receipt").get("countryCode"),
                "currency_code": subscription.get("data").get("receipt").get("priceCurrencyCode"),
                "transactionid": subscription["data"]["transactionId"],
                "purchaseDate": datetime.fromtimestamp(
                    float(subscription["data"]["purchaseDate"]) / 1000.0,
                    tz=get_current_timezone(),
                ),
                "expirationDate": datetime.fromtimestamp(
                    float(subscription["data"]["expirationDate"]) / 1000.0,
                    tz=get_current_timezone(),
                ),
                "meta": str(subscription["data"]),
            },
        )
        return user_subcription

class DownloadedSongsSerializer(serializers.ModelSerializer):
    song_list = serializers.SerializerMethodField()
    class Meta:
        model = DownloadedSongs
        fields = '__all__'
    
    def get_song_list(self, obj):
        serializer_context = {"request": self.context.get("request")}
        return SongsSerializerList(
            obj.songs.all(), many=True, context=serializer_context
        ).data

    def create(self, validated_data):
        obj, created = DownloadedSongs.objects.update_or_create(
            user=validated_data['user'], device_id=validated_data['device_id'],
            defaults={
                'user': validated_data['user'],
                'device_id': validated_data['device_id'],
                'device_type': validated_data['device_type'],
            },
        )
        songs = validated_data.get('songs',[])
        if len(songs) > 0:
            for song in songs:
                obj.songs.add(song)
        return obj