import base64
import json
import logging
from datetime import datetime, timedelta

import jwt
import pyotp
import requests, traceback
from background_task import background
from django.conf import settings
from django.forms import ValidationError
from fcm_django.models import FCMDevice

from MeditationApp.models import User, UserNotification

# from pyinapp import GooglePlayValidator, InAppValidationError
# from inapppy import GooglePlayValidator, InAppPyValidationError


logging.basicConfig(filename="debug.txt")
logger = logging.getLogger(__name__)

APPLE_PRODUCT_VERIFY_URL = "https://buy.itunes.apple.com/verifyReceipt"
APPLE_SANDBOX_VERIFY_URL = "https://sandbox.itunes.apple.com/verifyReceipt"
VERIFY_RECEIPT_STATUS = dict()
VERIFY_RECEIPT_STATUS[0] = "🎉 Receipt is valid"
VERIFY_RECEIPT_STATUS[21002] = "💣 The encoded receipt is malformed"
VERIFY_RECEIPT_STATUS[
    21004
] = "🔐The shared secret does not match what's on file with Apple"
VERIFY_RECEIPT_STATUS[21007] = "🔬 The receipt is from Sandbox. Add --use_sandbox"
VERIFY_RECEIPT_STATUS[21010] = "🖥 The receipt is from Production. Remove --use_sandbox"


def without_otp_otc_permission_jwt(user, session_key, device_id, device_type):
    payload = {
        "id": str(user.id) or "",
        "email": user.email or "",
        "jwt_secret": str(user.jwt_secret) or "",
        "session_key": session_key,
        'device_id': device_id,
        'device_type': device_type
    }
    jwt_token = jwt.encode(payload, settings.SECRET_KEY)
    logger.debug("[User logged In]" + str(jwt_token))
    return jwt_token


def permission_jwt(user, session_key, device_id, device_type):
    payload = {
        "id": str(user.id) or "",
        "email": user.email or "",
        "jwt_secret": str(user.jwt_secret) or "",
        "session_key": session_key,
        'device_id': device_id,
        'device_type': device_type
    }
    jwt_token = jwt.encode(payload, settings.SECRET_KEY)
    logger.debug("[User logged In]" + str(jwt_token))
    return jwt_token


def get_client_ip(request):
    x_forwarded_for = request.META.get("HTTP_X_FORWARDED_FOR")
    if x_forwarded_for:
        ip = x_forwarded_for.split(",")[0]
    else:
        ip = request.META.get("REMOTE_ADDR")
    return ip


def verify_apple_receipt(receipt_data):
    DEBUG = True
    receipt_json = json.dumps({"receipt-data": receipt_data}).encode("utf-8")
    verifyUrl = APPLE_PRODUCT_VERIFY_URL if not DEBUG else APPLE_SANDBOX_VERIFY_URL
    response = requests.request(
        method="POST",
        url=verifyUrl,
        headers={"content-type": "application/json"},
        data=receipt_json,
    )
    if response.status_code == 200:
        resJson = response.json()
        print(">>>>>>>>>>>>>>>>>>", resJson["status"])
        print("apple verify result: ", resJson)
        if resJson["status"] == 0:
            # verify success
            return True
    # failure
    return False


@background(schedule=60)
def notify_user_alarm_set_time(notification_data):
    try:
        # lookup user by id and send them a message
        notification_data["type"] = "music_alarm_notification"
        notification_data["subtype"] = "music_alarm_notification_start"
        # print(">>>>1", notification_data)
        user = notification_data.get("user_id")
        users = User.objects.filter(id=user)
        notification_devices = FCMDevice.objects.filter(
            user__in=users, active=True)
        if notification_devices:
            ios_devices = notification_devices.filter(type="ios")
            android_devices = notification_devices.filter(type="android")
            data = {
                "title": "Hey there! It's time to meditate",
                # "body": "Hey there! It's time to meditate",
                "body": "Tap on this notification to start the alarm",
                "sound": "default",
                "message": notification_data,
            }
            kwargs = {
                "content_available": True,
                "extra_kwargs": {
                    "priority": "high",
                    "mutable_content": True,
                    "notification": data,
                },
            }
            ios_stats = ios_devices.send_message(sound="default", **kwargs)
            android_stats = android_devices.send_message(
                data={
                    "title": "Hey there! It's time to meditate",
                    "body": notification_data,
                    "type": "music_alarm_notification",
                }
            )
            try:
                ios_success = ios_stats.get("success")
            except:
                ios_success = 0
            try:
                android_success = android_stats.get("success")
            except:
                android_success = 0
            if ios_success > 0 or android_success > 0:
                try:
                    try:
                        user = User.objects.get(id=user)
                    except:
                        user = None
                    if user:
                        UserNotification.objects.create(
                            user=user,
                            title="Alarm Notification",
                            message="Hey there! It's time to meditate",
                        )
                except:
                    pass
    except Exception as e:
        print("notify_user_alarm_set_time () ::", e)
        print("notify_user_alarm_set_time () :: traceback", traceback.format_exc())


@background(schedule=60)
def notify_user_alarm_end_time(notification_data):
    try:
        # lookup user by id and send them a message
        notification_data["type"] = "music_alarm_notification"
        notification_data["subtype"] = "music_alarm_notification_end"
        print(">>>>", notification_data)
        user = notification_data.get("user_id")
        users = User.objects.filter(id=user)
        notification_devices = FCMDevice.objects.filter(user__in=users)
        if notification_devices:
            # stats = notification_devices.send_message(
            #     data={
            #         "title":"music alarm notifiction end",
            #         "body":"music alarm notifiction end",
            #         "type":"music_alarm_notifiction"
            #     }
            # )
            # try:
            #     success = stats.get('success')
            # except:
            #     success = 0
            # print(">>>>",success, stats)
            ios_devices = notification_devices.filter(type="ios")
            android_devices = notification_devices.filter(type="android")
            data = {
                "title": "Hey there! This is time to stop",
                "body": "Tap on this notification to stop the alarm",
                "sound": "default",
                "message": notification_data,
            }
            kwargs = {
                "content_available": True,
                "extra_kwargs": {
                    "priority": "high",
                    "mutable_content": True,
                    "notification": data,
                },
            }
            ios_stats = ios_devices.send_message(sound="default", **kwargs)
            android_stats = android_devices.send_message(
                data={
                    "title": "Hey there! This is time to stop",
                    "body": notification_data,
                    "type": "music_alarm_notification",
                }
            )
            try:
                ios_success = ios_stats.get("success")
            except:
                ios_success = 0
            try:
                android_success = android_stats.get("success")
            except:
                android_success = 0
            print("ios music_alarm_notifiction stats", ios_success, ios_stats)
            print("android music_alarm_notifiction stats",
                android_success, android_stats)
            if ios_success > 0 or android_success > 0:
                try:
                    try:
                        user = User.objects.get(id=user)
                    except:
                        user = None
                    if user:
                        UserNotification.objects.create(
                            user=user,
                            title="Hey there! This is time to stop",
                            message="Alarm Notification",
                        )
                except:
                    pass
    except Exception as e:
        print("notify_user_alarm_end_time () ::", e)
        print("notify_user_alarm_end_time () :: traceback", traceback.format_exc())


@background(schedule=60)
def notify_user_alarm_duration_time(notification_data):
    try:
        # lookup user by id and send them a message
        notification_data["type"] = "music_duration_notification"
        user = notification_data.get("user_id")
        users = User.objects.filter(id=user)
        notification_devices = FCMDevice.objects.filter(user__in=users)
        if notification_devices:
            # stats = notification_devices.send_message(
            #     data={
            #         "title":"alarm duration notifiction",
            #         "body":"alarm duration notifiction",
            #         "type":"music_duration_notifiction"
            #     }
            # )
            # try:
            #     success = stats.get('success')
            # except:
            #     success = 0
            # print(">>>>",success, stats)
            ios_devices = notification_devices.filter(type="ios")
            android_devices = notification_devices.filter(type="android")
            data = {
                "title": "Alarm Duration Notification",
                "body": "Tap on this notification to finish the session",
                "sound": "default",
                "message": notification_data,
            }
            kwargs = {
                "content_available": True,
                "extra_kwargs": {
                    "priority": "high",
                    "mutable_content": True,
                    "notification": data,
                },
            }
            ios_stats = ios_devices.send_message(sound="default", **kwargs)
            android_stats = android_devices.send_message(
                data={
                    "title": "Alarm Duration Notification",
                    "body": notification_data,
                    "type": "music_duration_notification",
                }
            )
            try:
                ios_success = ios_stats.get("success")
            except:
                ios_success = 0
            try:
                android_success = android_stats.get("success")
            except:
                android_success = 0
            print("ios music_duration_notifiction stats", ios_success, ios_stats)
            print(
                "android music_duration_notifiction stats", android_success, android_stats
            )
            if ios_success > 0 or android_success > 0:
                try:
                    try:
                        user = User.objects.get(id=user)
                    except:
                        user = None
                    if user:
                        UserNotification.objects.create(
                            user=user,
                            title="Alarm Duration Notification",
                            message="Alarm Duration Notification",
                        )
                except:
                    pass
    except Exception as e:
        print("notify_user_alarm_duration_time () ::", e)
        print("notify_user_alarm_duration_time () :: traceback", traceback.format_exc())


def validate_subcription(obj):
    import json

    import requests

    url = "http://localhost:3000/users"

    payload = json.dumps(
        {
            "platform": obj["platform"],
            "receipt": obj["receipt"],
            "productId": obj["productid"],
            "packageName": obj["packageName"],
        }
    )
    headers = {"Content-Type": "application/json"}

    response = requests.request("GET", url, headers=headers, data=payload)

    response = json.loads(response.text)
    return response


def cancel_subcription(obj):
    import json

    import requests

    url = "http://localhost:3000/users/cancelSubscription"

    payload = json.dumps(
        {
            "platform": obj["platform"],
            "receipt": obj["receipt"],
            "productId": obj["productid"],
            "packageName": obj["packageName"],
        }
    )
    headers = {"Content-Type": "application/json"}

    response = requests.request("POST", url, headers=headers, data=payload)

    response = json.loads(response.text)
    return response

def notify_user_on_duplicate_login(notification_data):
    # lookup user by id and send them a message
    notification_data["type"] = "user_logout"
    notification_data["subtype"] = "user_logout"
    # print(">>>>1", notification_data)
    user = notification_data.get("user_email")
    device_id = notification_data.get("device_id")
    users = User.objects.filter(email=user)
    notification_devices = FCMDevice.objects.filter(
        user__in=users, active=True).exclude(device_id=device_id)

    if notification_devices:
        ios_devices = notification_devices.filter(type="ios")
        android_devices = notification_devices.filter(type="android")
        data = {
            "title": "Hey there! You have been logged in some other device.",
            "body": "Hey there! You have been logged in some other device.",
            "sound": "default",
            "message": notification_data,
        }
        kwargs = {
            "content_available": True,
            "extra_kwargs": {
                "priority": "high",
                "mutable_content": True,
                "notification": data,
            },
        }
        ios_stats = ios_devices.send_message(sound="default", **kwargs)
        android_stats = android_devices.send_message(
            data={
                "title": "Hey there! You have been logged in some other device.",
                "body": notification_data,
                "type": "user_logout",
            }
        )
        try:
            ios_success = ios_stats.get("success")
        except:
            ios_success = 0
        try:
            android_success = android_stats.get("success")
        except:
            android_success = 0
        if ios_success > 0 or android_success > 0:
            try:
                try:
                    user = User.objects.get(id=user)
                except:
                    user = None
                if user:
                    UserNotification.objects.create(
                        user=user,
                        title="Logout Notification",
                        message="Hey there! Hey there! You have been logged out.",
                    )
            except:
                pass
