import base64
import logging, traceback
import math
import uuid
from datetime import datetime, timedelta
import jwt
import facebook
import pyotp
import pytz
# from django.utils import six
import six
from django.conf import settings
from django.contrib import messages
from django.contrib.auth import authenticate, login
from django.contrib.auth import logout
from django.contrib.auth.hashers import make_password
from django.contrib.auth.tokens import PasswordResetTokenGenerator
from django.core.exceptions import ObjectDoesNotExist
from django.core.mail import EmailMessage
from django.shortcuts import render
from django.template.loader import render_to_string
from django.views.generic.edit import FormView
from django.db.models import Count
from django_filters import rest_framework as filters
from fcm_django.models import FCMDevice
from rest_framework import generics, permissions, viewsets
from rest_framework.decorators import permission_classes
from rest_framework.pagination import PageNumberPagination
from rest_framework.permissions import AllowAny, IsAuthenticated
from rest_framework.response import Response
from rest_framework.status import HTTP_400_BAD_REQUEST
from rest_framework.views import APIView
from background_task.models import Task, CompletedTask
from MeditationApp.backends import JWTAuthentication
from MeditationApp.filters import (EffectCategoryFilter, PageFilter,
                                   PersonlizeFilter, SongsCategoryFilter,
                                   SongsFilter, UserFavouriteFilter,
                                   UserListFilter)
from MeditationApp.models import (OTP, DownloadedSongs, EffectCategory,
                                  EffectSongs, ExploreSongs, LoginPageVideo,
                                  Page, Personlize, Recommendation, Songs,
                                  SongsCategory, TodaySpecial,
                                  User, UserFavourite, UserNotification,
                                  UserSubscription, UserVideoFavourite)
from MeditationApp.serializers import (AddAlarmNotificationSerializer,
                                       AddDurationNotificationSerializer,
                                       ChangePasswordSerializer,
                                       ConfirmTokenSerializer,
                                       ContactUsSerializer, DownloadedSongsSerializer,
                                       EffectCategorySerializerList,
                                       EffectSongsSerializerList,
                                       ExploreSongsList,
                                       ForgotPasswordSerializer,
                                       LoginPageVideoSerializerList,
                                       LoginSerializer, PageSerializerList,
                                       PersonlizeSerializer,
                                       ResetPasswordSerializer,
                                       SocialAuthAppleSerializer,
                                       SocialAuthGoogleSerializer,
                                       SocialAuthSerializer,
                                       SongsCategorySerializerList,
                                       SongsSerializerList,
                                       SubscriptionReceiptSerializer,
                                       TodaySpecialSerializerList,
                                       UpdateTodayspecialVideoSerializer,
                                       UserFavouriteSerializer,
                                       UserNotificationSerializerList,
                                       UserSerializer, UserSerializerList,
                                       UserUpdateSerializer, UserVideoFavouriteSerializer)
from MeditationApp.utils import (notify_user_alarm_duration_time,
                                 notify_user_alarm_end_time,
                                 notify_user_alarm_set_time, notify_user_on_duplicate_login, permission_jwt,
                                 without_otp_otc_permission_jwt)

from .forms import SetPasswordForm
from dateutil import tz

logging.basicConfig(filename="debug.txt")
logger = logging.getLogger(__name__)


class CustomPagination(PageNumberPagination):
    permission_classes = (IsAuthenticated,)
    authentication_classes = (JWTAuthentication,)
    """
	bank transaction pahination

	"""
    page_size = 10
    page_size_query_param = "page_size"
    max_page_size = 1000

    def get_paginated_response(self, data):
        try:
            count = self.page.paginator.count
            pages = self.page_size
            p = count / pages
            page_size = math.trunc(p)
        except:
            page_size = 0

        return Response(
            {
                "links": {
                    "next": self.get_next_link(),
                    "previous": self.get_previous_link(),
                },
                "count": self.page.paginator.count,
                "page_size": page_size + 1,
                "results": data,
            }
        )


class AccountActivationTokenGenerator(PasswordResetTokenGenerator):
    def _make_hash_value(self, user, timestamp):
        return (
            six.text_type(user.pk)
            + six.text_type(timestamp)
            + six.text_type(user.is_active)
        )


account_activation_token = AccountActivationTokenGenerator()


class generateKey:
    @staticmethod
    def returnValue(id):
        return str(id) + str(datetime.date(datetime.now())) + settings.SECRET_KEY


def get_otp(user):
    keygen = generateKey()
    key = base64.b32encode(keygen.returnValue(
        user.id).encode())  # Key is generated
    return pyotp.HOTP(key)  # HOTP Model for OTP is created


class UserViewSet(viewsets.ModelViewSet):
    permission_classes = (IsAuthenticated,)
    authentication_classes = (JWTAuthentication,)

    queryset = User.objects.all()

    def get_serializer_class(self):
        if self.action == "list":
            return UserSerializerList
        if self.action == "retrieve":
            return UserSerializerList
        if self.action == "update":
            return UserUpdateSerializer
        return UserSerializer

    filter_backends = (filters.DjangoFilterBackend,)
    filterset_class = UserListFilter

    def list(self, request, *args, **kwargs):
        try:
            query_object = self.filter_queryset(self.get_queryset())
            serializer = self.get_serializer(instance=query_object, many=True)
            if serializer.data:
                user = serializer.data[0]
            else:
                user = {}
            response = {
                "message": "user data fetched successfully",
                "code": 200,
                "data": {"user": user},
            }
            return Response(response, status=200)
        except Exception as e:
            message = "Some error is occured"
            response = {"message": message, "code": 400, "data": {}}
            return Response(response, status=400)

    def retrieve(self, request, *args, **kwargs):
        try:
            query_object = self.get_object()
            serializer = self.get_serializer(instance=query_object)
            response = {
                "message": "user data fetched successfully",
                "code": 200,
                "data": serializer.data,
            }
            return Response(response, status=200)
        except Exception as e:
            message = "Some error is occured"
            response = {"message": message, "code": 400, "data": {}}
            return Response(response, status=400)

    def update(self, request, *args, **kwargs):
        partial = True
        instance = self.get_object()
        data = request.data
        serializer = self.get_serializer(
            instance=instance, data=data, partial=partial)
        if serializer.is_valid(raise_exception=True):
            self.perform_update(serializer)
            response = {
                "valid": True,
                "message": " User data Updated successfully",
                "code": 200,
                "data": {"user": serializer.data},
            }
            return Response(response, status=200)

        response = {"valid": False, "code": 400, "message": serializer.errors}
        return Response(response, status=400)


@permission_classes((permissions.AllowAny,))
class SignUpViewSet(generics.CreateAPIView):

    serializer_class = UserSerializer

    def create(self, request, *args, **kwarg):
        res = {}
        data = request.data
        serializer = self.get_serializer(data=request.data)
        if serializer.is_valid():
            serializer.save(password=make_password(
                request.data.get("password")))
            user = User.objects.get(id=serializer.data.get("id"))
            response = {
                "code": 200,
                "data": {},
                "message": "Sign up created Successfully",
                "id": str(user.id),
                "username": user.username,
            }
            return Response(response, status=200)
        else:
            try:
                message = list(serializer.errors.values())[0][0]
            except:
                message = "some error is occured please try again"
            response = {"code": 400, "message": message}

            return Response(response, status=400)


@permission_classes((permissions.AllowAny,))
class LoginView(generics.GenericAPIView):
    """
    login view to login User
    """

    serializer_class = LoginSerializer

    def post(self, request):
        data = {}
        response = {}
        if not request.data:
            response["status"] = "Please provide email/password"
            return Response(response, status=HTTP_400_BAD_REQUEST)
        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():
            email = request.data.get("email")
            password = request.data.get("password")
            device_id = request.data.get("device_id")
            device_token = request.data.get("device_token")
            device_type = request.data.get("device_type")
            flag=False
            if email and password:

                # validate the email
                if not User.objects.filter(email=email).exists():
                    response = {
                        "code": 400,
                        "message": "There is no user found of this email, Please sign up.",
                    }
                    return Response(response, status=400)
                
                
                        

                # authenticate the user account with email and password.
                user = authenticate(email=email, password=password)
                user_token_dev = FCMDevice.objects.filter(user=user)
                # print(request.user)
                if user_token_dev.exists():
                    notification_data = {
                        "user_email": email,
                        "device_id": device_id,
                        "message": "You have been logged in from other device."
                    }

                if user:
                    if user.is_active:
                        data["user"] = user
                        if device_type == "android" or device_type == "ios":
                            if user_token_dev.exists():
                                notify_user_on_duplicate_login(notification_data)
                                user_token_dev.delete()
                                Task.objects.filter(queue=user.id).delete()
                                CompletedTask.objects.filter(queue=user.id).delete()

                            FCMDevice.objects.update_or_create(device_id=device_id,
                                               defaults={
                                                   "name": user.username,
                                                   "user": user,
                                                   "device_id": device_id,
                                                   "registration_id": device_token,
                                                   "type": device_type,
                                                   "active": True
                                               })
                    else:
                        response["message"] = "User is deactivated."
                        response = {"code": 400,
                                    "message": "User is deactivated."}
                        return Response(response, status=400)
                else:
                    response = {
                        "code": 400,
                        "message": "You have entered wrong email or password.",
                    }
                    return Response(response, status=400)
            else:
                response = {
                    "code": 400,
                    "message": "Must provide email and password both.",
                }
                return Response(response, status=400)
                category_id = None
            if user.is_superuser != True:
                jwt_token = permission_jwt(user, request.session.session_key,device_id, device_type)
                response = {
                    "code": 200,
                    "data": {"token": jwt_token},
                    "message": "Logged In Successfully",
                    "id": user.id,
                    "username": user.username,
                    "jwt_secret": str(user.jwt_secret),
                    "check_subscription": user.check_trail_subscription,
                    "cancel_subscription_applied": user.cancel_subscription_applied,
                    "download_songs_count": user.download_songs_count,
                }
                return Response(response, status=200)
            else:
                jwt_token = without_otp_otc_permission_jwt(
                    user, request.session.session_key,device_id, device_type
                )
                response = {
                    "code": 200,
                    "data": {"token": jwt_token},
                    "message": "Logged In Successfully",
                    "id": user.id,
                    "username": user.username,
                    "jwt_secret": str(user.jwt_secret),
                    "check_subscription": True,
                    "download_songs_count": user.download_songs_count,
                }
                return Response(response, status=200)
        else:
            response = {
                "code": 400,
                "message": "Unable to login using these credentials",
            }
            return Response(response, status=400)


class LogoutView(APIView):
    permission_classes = (IsAuthenticated,)
    authentication_classes = (JWTAuthentication,)

    def post(self, request):
        try:
            user = request.user
            user.jwt_secret = uuid.uuid4()
            user.save()      
            FCMDevice.objects.filter(user=user).delete()  
            Task.objects.filter(queue=user.id).delete()
            CompletedTask.objects.filter(queue=user.id).delete()    
            logout(request)
            response = {"code": 200, "message": "Logged out successfully"}
            return Response(response, status=200)
        except:
            message = "Something went wrong"
            response = {"message": message, "code": 400, "data": {}}
            return Response(response, status=400)


class ChangePasswordView(generics.UpdateAPIView):
    """
    An endpoint for changing password.
    """

    serializer_class = ChangePasswordSerializer
    permission_classes = (IsAuthenticated,)
    authentication_classes = (JWTAuthentication,)

    def get_object(self, queryset=None):
        obj = self.request.user
        return obj

    def update(self, request, *args, **kwargs):
        try:
            self.object = self.get_object()
            serializer = self.get_serializer(data=request.data)

            if serializer.is_valid():
                old_password = serializer.data.get("old_password")
                user = self.request.user
                if user.check_password(old_password):

                    # set_password also hashes the password that the user will get
                    self.object.set_password(
                        serializer.data.get("new_password"))
                    self.object.save()
                    response = {
                        "valid": True,
                        "code": 200,
                        "message": "Password changed successfully",
                    }
                    return Response(response, status=200)
                else:
                    response = {
                        "valid": False,
                        "code": 400,
                        "message": "Current password does not match",
                    }
                    return Response(response, status=400)
            else:
                response = {
                    "valid": False,
                    "code": 400,
                    "message": "Something went wrong",
                }
                return Response(response, status=400)
        except Exception as e:
            response = {"valid": False, "code": 400,
                        "message": "Something went wrong"}
            return Response(response, status=400)


class ForgotPasswordView(generics.GenericAPIView):
    permission_classes = [AllowAny]
    serializer_class = ForgotPasswordSerializer
    authentication_classes = []

    def post(self, request, *args):
        try:
            ip_address = "api.souljourney.io"
            email = request.data.get("email")
            try:
                user = User.objects.get(email=email)
            except ObjectDoesNotExist:
                user = None
            if not user:
                message = "User Not Found"
                response = {"valid": False, "message": message,
                            "code": 400, "data": {}}
                return Response(response, status=400)
            try:
                if user:
                    subject = "Password Reset Account"
                    message = render_to_string(
                        "account_activation_email.html",
                        {
                            "user": user,
                            "domain": ip_address,
                            # urlsafe_base64_encode(force_bytes(user.pk)),
                            "uid": user.pk,
                            "token": account_activation_token.make_token(user),
                        },
                    )
                    to_email = user.email
                    email = EmailMessage(subject, message, to=[to_email])
                    email.send()
                response = {"message": "OTP sent successfully",
                            "code": 200, "data": {}}
                return Response(response, status=200)
            except Exception as e:
                message = "Something went wrong"
                response = {"message": message, "code": 400, "data": {}}
                return Response(response, status=400)
        except:
            message = "Something went wrong"
            response = {"message": message, "code": 400, "data": {}}
            return Response(response, status=400)


class ConfirmTokenView(generics.GenericAPIView):
    permission_classes = [AllowAny]
    serializer_class = ConfirmTokenSerializer

    def post(self, request):
        email = request.data.get("email")
        try:
            user = User.objects.get(email=email)
        except ObjectDoesNotExist:
            user = None
        if not user:
            message = "User Not Found"
            response = {"message": message, "code": 400, "data": {}}
            return Response(response, status=400)
        try:
            serializer = self.serializer_class(data=request.data)
            if serializer.is_valid():
                token = serializer.data.get("token")
                try:
                    otpObj = OTP.objects.get(
                        user=user
                    )  # if Mobile already exists the take this else create New One
                except ObjectDoesNotExist:
                    otpObj = None
                otp = get_otp(user)
                if otp.verify(token, otpObj.counter):
                    # user.set_password(serializer.data.get("new_password"))
                    # user.save()
                    response = {"message": "Token is correct",
                                "code": 200, "data": []}
                    return Response(response, status=200)
                else:
                    message = "OTP is incorrect"
                    response = {"message": message, "code": 400, "data": {}}
                    return Response(response, status=400)
            else:
                message = "Please enter OTP and Password"
                response = {"message": message, "code": 400, "data": {}}
                return Response(response, status=400)
        except:
            message = "Some error is occured Please try again"
            response = {"message": message, "code": 400, "data": {}}
            return Response(response, status=400)


class ResetPasswordView(generics.GenericAPIView):
    permission_classes = [AllowAny]
    """
		An endpoint for reset password.
		"""
    serializer_class = ResetPasswordSerializer

    def update(self, request):
        email = request.data.get("email")
        try:
            user = User.objects.get(email=email)
        except:
            user = None
        if not user:
            message = "User Not Found"
            response = {"message": message, "code": 400, "data": {}}
            return Response(response, status=400)
        # self.object = self.get_object()
        serializer = self.get_serializer(data=request.data)
        if serializer.is_valid():
            # set_password also hashes the password that the user will get
            user.set_password(serializer.data.get("password"))
            user.save()
            response = {
                "code": 200,
                "message": "Password updated successfully",
                "data": [],
            }

            return Response(response, status=200)
        response = {"message": serializer.errors, "data": [], "code": 400}
        return Response(response, status=400)


class PasswordResetConfirmView(FormView):
    template_name = "password_reset.html"
    success_url = "/api/v1/success"
    form_class = SetPasswordForm

    def post(self, request, uidb64=None, token=None, *arg, **kwargs):
        """
        View that checks the hash in a password reset link and presents a
        form for entering a new password.
        """

        form = self.form_class(request.POST)
        assert uidb64 is not None and token is not None  # checked by URLconf
        try:
            uid = uidb64
            user = User.objects.get(pk=uid)
        except:
            user = None
        if user is not None and account_activation_token.check_token(user, token):
            if form.is_valid():
                new_password = form.cleaned_data["new_password2"]
                user.set_password(new_password)
                user.save()
                messages.success(request, "Password has been reset.")
                return self.form_valid(form)
            else:
                messages.error(
                    request, "Password reset has not been unsuccessful.")
                return self.form_invalid(form)
        else:
            messages.error(
                request, "The reset password link is no longer valid.")
            return self.form_invalid(form)


def success_view(request):
    return render(request, "sucess.html")


class PersonlizeViewSet(generics.ListCreateAPIView):
    permission_classes = (IsAuthenticated,)
    authentication_classes = (JWTAuthentication,)

    queryset = Personlize.objects.filter(isDeleted=False).all()
    serializer_class = PersonlizeSerializer
    filter_backends = (filters.DjangoFilterBackend,)
    filterset_class = PersonlizeFilter

    def list(self, request, *args, **kwargs):
        try:
            queryset = self.filter_queryset(self.get_queryset())
            serializer = self.get_serializer(instance=queryset, many=True)
            response = {
                "valid": True,
                "code": 200,
                "message": "Data fetch successfully",
                "data": serializer.data,
            }
            return Response(response, status=200)
        except Exception as e:
            message = "Some error is occured"
            response = {"valid": False, "message": message,
                        "code": 400, "data": {}}
            return Response(response, status=400)

    def create(self, request, *args, **kwargs):
        try:
            data = request.data
            recommendation = data.pop("recommendation")
            try:
                user = User.objects.get(id=data.get("user"))
            except:
                user = None
            if user:
                personlize = Personlize.objects.create(user=user)
                for name in recommendation:
                    recommendation = Recommendation.objects.create(name=name)
                    personlize.recommendation.add(recommendation)
                response = {
                    "message": "user personlize data created successfully",
                    "code": 200,
                    "data": {"personlize_id": personlize.id},
                }
                return Response(response, status=200)
            else:
                message = "User does not exists"
                response = {"message": message, "code": 400, "data": {}}
                return Response(response, status=400)

        except Exception as e:
            message = "Some error is occured"
            response = {"message": message, "code": 400, "data": {}}
            return Response(response, status=400)

    def user_personlize_create(self, serializer):
        save_data = serializer.save()


class DashboardListView(generics.ListAPIView):
    permission_classes = (IsAuthenticated,)
    authentication_classes = (JWTAuthentication,)

    def get_queryset(self):
        queryset = self.request.user
        return queryset

    serializer_class = UserSerializerList

    def list(self, request, *args, **kwargs):
        try:
            queryset = self.get_queryset()
            serializer = self.get_serializer(
                queryset)  # TodaySpecialSerializerList
            todaySpecial = TodaySpecial.objects.filter(isDeleted=False, status=True).first()
            todaySpecialData = TodaySpecialSerializerList(
                instance=todaySpecial).data
            mostPopularObj = Songs.objects.filter(
                isDeleted=False, status=True,
                created__lte=datetime.today(),
                created__gt=datetime.today() - timedelta(days=30),
            ).order_by("listen_count")[:10]
            mostPopularSongs = SongsSerializerList(
                instance=mostPopularObj, many=True, context={"request": self.request}
            ).data
            songsCategoryObj = SongsCategory.objects.filter(
                isDeleted=False, status=True, name="meditation"
            ).first()
            meditationSongsCategoryData = SongsCategorySerializerList(
                instance=songsCategoryObj, context={"request": self.request}
            ).data
            response = {
                "valid": True,
                "code": 200,
                "message": "Data fetch successfully",
                "data": {
                    "user": serializer.data,
                    "meditation": meditationSongsCategoryData,
                    "most_popular_songs": mostPopularSongs,
                    "today_special": todaySpecialData,
                },
            }
            return Response(response, status=200)
        except Exception as e:
            message = "Some error is occured"
            response = {"valid": False, "message": message,
                        "code": 400, "data": {}}
            return Response(response, status=400)


class SearchSongsListView(generics.ListAPIView):
    permission_classes = (IsAuthenticated,)
    authentication_classes = (JWTAuthentication,)
    queryset = Songs.objects.filter(isDeleted=False, status=True).all()
    serializer_class = SongsSerializerList
    filter_backends = (filters.DjangoFilterBackend,)
    filterset_class = SongsFilter

    def list(self, request, *args, **kwargs):
        try:
            queryset = self.filter_queryset(self.get_queryset())
            serializer = self.get_serializer(
                instance=queryset, many=True, context={"request": self.request}
            )
            response = {
                "valid": True,
                "code": 200,
                "message": "Data fetch successfully",
                "data": {"songs": serializer.data},
            }
            return Response(response, status=200)
        except:
            message = "Some error is occured"
            response = {"valid": False, "message": message,
                        "code": 400, "data": {}}
            return Response(response, status=400)


class SongsListView(generics.ListAPIView):
    permission_classes = (IsAuthenticated,)
    authentication_classes = (JWTAuthentication,)
    queryset = SongsCategory.objects.filter(isDeleted=False, status=True).all()
    serializer_class = SongsCategorySerializerList
    filter_backends = (filters.DjangoFilterBackend,)
    filterset_class = SongsCategoryFilter

    def get_serializer_context(self):
        context = super(SongsListView, self).get_serializer_context()
        context.update({"request": self.request})
        return context

    def list(self, request, *args, **kwargs):
        try:
            queryset = self.filter_queryset(self.get_queryset())
            serializer = self.get_serializer(instance=queryset, many=True)
            user = self.request.user
            if user.listing_status:
                status = "locked"
            else:
                status = "unlocked"
            response = {
                "valid": True,
                "code": 200,
                "message": "Data fetch successfully",
                "data": {
                    "category": serializer.data,
                    "user_id": user.id,
                    "listening_status": status,
                },
            }
            return Response(response, status=200)
        except:
            message = "Some error is occured"
            response = {"valid": False, "message": message,
                        "code": 400, "data": {}}
            return Response(response, status=400)


class UserUpdateView(generics.UpdateAPIView):
    permission_classes = (IsAuthenticated,)
    authentication_classes = (JWTAuthentication,)
    """
		An endpoint for update user.
		"""
    serializer_class = UserUpdateSerializer
    permission_classes = (IsAuthenticated,)
    authentication_classes = (JWTAuthentication,)

    def get_object(self, queryset=None):
        obj = self.request.user
        return obj

    def update(self, request, *args, **kwargs):
        try:
            partial = True
            instance = self.get_object()
            data = request.data
            email = request.data.get("email", "")
            if email != instance.email:
                try:
                    is_email = User.objects.get(email=email)
                except:
                    is_email = None
                if is_email:
                    message = "Email already exist"
                    response = {
                        "valid": False,
                        "message": message,
                        "code": 400,
                        "data": {},
                    }
                    return Response(response, status=400)

            serializer = self.get_serializer(
                instance=instance, data=data, partial=partial
            )
            if serializer.is_valid():
                self.perform_update(serializer)
                response = {
                    "valid": True,
                    "message": " User data Updated successfully",
                    "code": 200,
                    "data": {"user": serializer.data},
                }
                return Response(response, status=200)

            response = {"valid": False, "code": 400,
                        "message": serializer.errors}
            return Response(response, status=400)
        except Exception as e:
            print(">>>>>>>>>", e)
            message = "Some error is occured"
            response = {"valid": False, "message": message,
                        "code": 400, "data": {}}
            return Response(response, status=400)


class SongAddToFavouriteViewSet(generics.CreateAPIView):
    serializer_class = UserFavouriteSerializer

    def create(self, request, *args, **kwarg):
        try:
            data = request.data
            serializer = self.get_serializer(data=request.data)
            if serializer.is_valid():
                serializer.save()
                response = {
                    "valid": True,
                    "code": 200,
                    "message": "Song added in wishlist successfully",
                    "data": {},
                }
                return Response(response, status=200)
            else:
                try:
                    message = list(serializer.errors.values())[0][0]
                except:
                    message = "some error is occured please try again"
                response = {"valid": False, "code": 400,
                            "message": message, "data": {}}

                return Response(response, status=400)
        except Exception as e:
            message = "Some error is occured"
            response = {"valid": False, "message": message,
                        "code": 400, "data": {}}
            return Response(response, status=400)


class VideoAddToFavouriteViewSet(generics.CreateAPIView):
    serializer_class = UserVideoFavouriteSerializer

    def create(self, request, *args, **kwarg):
        try:
            data = request.data
            serializer = self.get_serializer(data=request.data)
            if serializer.is_valid():
                serializer.save()
                response = {
                    "valid": True,
                    "code": 200,
                    "message": "Video added in wishlist successfully",
                    "data": {},
                }
                return Response(response, status=200)
            else:
                try:
                    message = list(serializer.errors.values())[0][0]
                except:
                    message = "some error is occured please try again"
                response = {"valid": False, "code": 400,
                            "message": message, "data": {}}

                return Response(response, status=400)
        except Exception as e:
            message = "Some error is occured"
            response = {"valid": False, "message": message,
                        "code": 400, "data": {}}
            return Response(response, status=400)


class FavouriteSongListViewSet(generics.ListAPIView):
    permission_classes = (IsAuthenticated,)
    authentication_classes = (JWTAuthentication,)
    queryset = UserFavourite.objects.filter(isDeleted=False, status=True).all()
    serializer_class = SongsSerializerList
    filter_backends = (filters.DjangoFilterBackend,)
    filterset_class = UserFavouriteFilter

    def list(self, request, *args, **kwargs):
        try:
            queryset = self.filter_queryset(self.get_queryset())
            songs_ids = queryset.values_list("songs__id", flat=True)
            songQueryset = Songs.objects.filter(
                id__in=songs_ids, isDeleted=False, status=True).all()
            serializer = self.get_serializer(
                instance=songQueryset, many=True, context={"request": self.request}
            )
            response = {
                "valid": True,
                "code": 200,
                "message": "Data fetch successfully",
                "data": {"songs": serializer.data},
            }
            return Response(response, status=200)
        except:
            message = "Some error is occured"
            response = {"valid": False, "message": message,
                        "code": 400, "data": {}}
            return Response(response, status=400)


class UserFavouriteRemoveViewSet(generics.GenericAPIView):
    permission_classes = (IsAuthenticated,)
    authentication_classes = (JWTAuthentication,)
    serializer_class = UserFavouriteSerializer

    def post(self, request, *args, **kwargs):
        try:
            data = request.data
            song_id = data.get("songs")
            user = data.get("user")
            try:
                userFavourite = UserFavourite.objects.filter(
                    user=user, songs=song_id, isDeleted=False, status=True
                )
            except:
                userFavourite = None
            if userFavourite:
                userFavourite.delete()
                response = {
                    "valid": True,
                    "code": 200,
                    "message": "Data delete successfully",
                    "data": {},
                }
                return Response(response, status=200)
            else:
                message = "Song is not present favourites"
                response = {"valid": False, "message": message,
                            "code": 400, "data": {}}
                return Response(response, status=400)

        except Exception as e:
            message = "Some error is occured"
            response = {"valid": False, "message": message,
                        "code": 400, "data": {}}
            return Response(response, status=400)


class UserFavouriteVideoRemoveViewSet(generics.GenericAPIView):
    permission_classes = (IsAuthenticated,)
    authentication_classes = (JWTAuthentication,)
    serializer_class = UserVideoFavouriteSerializer

    def post(self, request, *args, **kwargs):
        try:
            data = request.data
            videos = data.get("videos")
            user = data.get("user")
            userfavouritevideo = userFavouriteVideo = UserVideoFavourite.objects.filter(
                    user=user, videos__id=videos
                )
            if userfavouritevideo.exists():
                userfavouritevideo.delete()
                response = {
                    "valid": True,
                    "code": 200,
                    "message": "Data delete successfully",
                    "data": {},
                }
                return Response(response, status=200)
            else:
                message = "Video is not present favourites"
                response = {"valid": False, "message": message,
                            "code": 400, "data": {}}
                return Response(response, status=400)

        except Exception as e:
            message = "Some error is occured"
            response = {"valid": False, "message": message,
                        "code": 400, "data": {}}
            return Response(response, status=400)


class FacebookView(generics.GenericAPIView):
    permission_classes = [AllowAny]
    serializer_class = SocialAuthSerializer

    def post(self, request):
        access_token = request.data.get("token")  # validate the token
        serializer = self.serializer_class(data=request.data)
        if serializer.is_valid():
            token = serializer.data.get("token")
            device_id = serializer.data.get("device_id")
            device_token = serializer.data.get("device_token")
            device_type = serializer.data.get("device_type")
            try:
                graph = facebook.GraphAPI(access_token=access_token)
                user_info = graph.get_object(
                    id="me",
                    fields="first_name, middle_name, last_name, id, "
                    "currency, hometown, location, locale, "
                    "email, gender, interested_in, picture.type(large),"
                    " birthday, cover",
                )
            except facebook.GraphAPIError:
                message = "Invalid Token"
                response = {"valid": False, "message": message,
                            "code": 400, "data": {}}
                return Response(response, status=201)
            try:
                user = User.objects.get(email=user_info.get("email"))
            except User.DoesNotExist:
                password = User.objects.make_random_password()
                user = User(
                    firstname=user_info.get("first_name"),
                    email=user_info.get("email"),
                    image=user_info.get("picture")["data"]["url"],
                    username=user_info.get("username"),
                )
                user.set_password(password)
                user.save()
            try:
                try:
                    FCMDevice.objects.filter(device_id=device_id).delete()
                except:
                    pass
                FCMDevice.objects.create(
                    name=user.username,
                    user=user,
                    device_id=device_id,
                    registration_id=device_token,
                    type=device_type,
                    active=True,
                )
            except:
                pass
            try:
                jwt_token = permission_jwt(user, request.session.session_key)
                response = {
                    "code": 200,
                    "data": {"token": jwt_token},
                    "message": "Logged In Successfully",
                    "id": user.id,
                    "username": user.username,
                    "jwt_secret": str(user.jwt_secret),
                    "check_subscription": user.check_trail_subscription,
                    "cancel_subscription_applied": user.cancel_subscription_applied,
                }
                return Response(response, status=200)
            except:
                message = "Some error is occured"
                response = {"valid": False, "message": message,
                            "code": 400, "data": {}}
                return Response(response, status=400)
        else:
            response = {
                "valid": False,
                "message": serializer.errors,
                "code": 400,
                "data": {},
            }
            return Response(response, status=400)


class GoogleView(generics.GenericAPIView):
    permission_classes = [AllowAny]
    serializer_class = SocialAuthGoogleSerializer

    def post(self, request):
        try:
            name = request.data.get("name")
            email = request.data.get("email")
            username = request.data.get("username")
            device_id = request.data.get("device_id")
            device_token = request.data.get("device_token")
            device_type = request.data.get("device_type")
            try:
                user = User.objects.get(email=email)
            except User.DoesNotExist:
                password = User.objects.make_random_password()
                user = User()
                user.username = username
                user.firstname = name
                # provider random default password
                user.password = password
                user.email = email
                user.save()

            FCMDevice.objects.update_or_create(device_id=device_id,
                                               defaults={
                                                   "name": user.username,
                                                   "user": user,
                                                   "device_id": device_id,
                                                   "registration_id": device_token,
                                                   "type": device_type,
                                                   "active": True
                                               })

            jwt_token = permission_jwt(
                user, request.session.session_key, device_id, device_type)
            response = {
                "code": 200,
                "data": {"token": jwt_token},
                "message": "Logged In Successfully",
                "id": user.id,
                "username": user.username,
                "jwt_secret": str(user.jwt_secret),
                "check_subscription": user.check_trail_subscription,
                "cancel_subscription_applied": user.cancel_subscription_applied,
                "download_songs_count": user.download_songs_count,
            }
            return Response(response, status=200)
        except Exception as e:
            message = "Some error is occured"
            response = {"valid": False, "message": message,
                        "code": 400, "data": {}}
            return Response(response, status=400)


class SigninAppleView(generics.GenericAPIView):
    permission_classes = [AllowAny]
    serializer_class = SocialAuthAppleSerializer

    def post(self, request):
        try:
            name = request.data.get("name")
            email = request.data.get("email")
            username = request.data.get("username")
            device_id = request.data.get("device_id")
            device_token = request.data.get("device_token")
            device_type = request.data.get("device_type")
            try:
                user = User.objects.get(email=email)
            except:
                password = User.objects.make_random_password()
                user = User()
                user.username = username
                user.firstname = name
                # provider random default password
                user.password = password
                user.email = email
                user.save()

            FCMDevice.objects.update_or_create(device_id=device_id,
                                               defaults={
                                                   "name": user.username,
                                                   "user": user,
                                                   "device_id": device_id,
                                                   "registration_id": device_token,
                                                   "type": device_type,
                                                   "active": True
                                               })
            jwt_token = permission_jwt(user, request.session.session_key, device_id, device_type)
            response = {
                "code": 200,
                "data": {"token": jwt_token},
                "message": "Logged In Successfully",
                "id": user.id,
                "jwt_secret": str(user.jwt_secret),
                "check_subscription": user.check_trail_subscription,
                "cancel_subscription_applied": user.cancel_subscription_applied,
                "download_songs_count": user.download_songs_count,
            }
            return Response(response, status=200)
        except Exception as e:
            message = "Some error is occured"
            response = {"valid": False, "message": message,
                        "code": 400, "data": {}}
            return Response(response, status=400)


class EffectCategoryListView(generics.ListAPIView):
    permission_classes = (IsAuthenticated,)
    authentication_classes = (JWTAuthentication,)
    queryset = EffectCategory.objects.filter(isDeleted=False, status=True).all()
    serializer_class = EffectCategorySerializerList
    filter_backends = (filters.DjangoFilterBackend,)
    filterset_class = EffectCategoryFilter

    def get_serializer_context(self):
        context = super(EffectCategoryListView, self).get_serializer_context()
        context.update({"request": self.request})
        return context

    def list(self, request, *args, **kwargs):
        try:
            queryset = self.filter_queryset(self.get_queryset())
            serializer = self.get_serializer(instance=queryset, many=True)
            response = {
                "valid": True,
                "code": 200,
                "message": "Data fetch successfully",
                "data": {"effect_data": serializer.data},
            }
            return Response(response, status=200)
        except Exception as e:
            message = "Some error is occured"
            response = {"valid": False, "message": message,
                        "code": 400, "data": {}}
            return Response(response, status=400)


class ExploreSongsListView(generics.ListAPIView):
    permission_classes = (IsAuthenticated,)
    authentication_classes = (JWTAuthentication,)
    queryset = ExploreSongs.objects.filter(isDeleted=False, status=True)
    serializer_class = ExploreSongsList

    def get_serializer_context(self):
        context = super(ExploreSongsListView, self).get_serializer_context()
        context.update({"request": self.request})
        return context

    def list(self, request, *args, **kwargs):
        try:
            queryset = self.get_queryset()
            serializer = self.get_serializer(
                instance=queryset, context={"request": self.request}
            )
            response = {
                "valid": True,
                "code": 200,
                "message": "Data fetch successfully",
                "data": {"effect_data": serializer.data},
            }
            return Response(response, status=200)
        except Exception as e:
            message = "Some error is occured"
            response = {"valid": False, "message": message,
                        "code": 400, "data": {}}
            return Response(response, status=400)


class SearchCategoryListView(generics.ListAPIView):
    permission_classes = (IsAuthenticated,)
    authentication_classes = (JWTAuthentication,)
    queryset = SongsCategory.objects.filter(isDeleted=False, status=True).all()
    serializer_class = SongsCategorySerializerList

    def get_serializer_context(self):
        context = super(SearchCategoryListView, self).get_serializer_context()
        context.update({"request": self.request})
        return context

    def list(self, request, *args, **kwargs):
        try:
            queryset = self.get_queryset().filter(status=True, isDeleted=False)           
           
            response = {
                "valid": True,
                "code": 200,
                "message": "Data fetch successfully",
                "data": {"category": self.get_serializer(
                    instance=queryset, many=True).data},
            }
            return Response(response, status=200)
        except Exception as e:
            message = "Some error is occured"
            response = {"valid": False, "message": message,
                        "code": 400, "data": {}}
            return Response(response, status=400)


class BinauralEffectSongsListView(generics.ListAPIView):
    permission_classes = (IsAuthenticated,)
    authentication_classes = (JWTAuthentication,)
    queryset = EffectSongs.objects.filter(isDeleted=False, status=True).all()
    serializer_class = EffectSongsSerializerList

    def get_serializer_context(self):
        context = super(BinauralEffectSongsListView,
                        self).get_serializer_context()
        context.update({"request": self.request})
        return context

    def list(self, request, *args, **kwargs):
        try:
            queryset = self.get_queryset().filter(status=True, isDeleted=False)           
            response = {
                "valid": True,
                "code": 200,
                "message": "Data fetch successfully",
                "data": {"effect_songs": self.get_serializer(instance=queryset, many=True).data},
            }
            return Response(response, status=200)
        except Exception as e:
            message = "Some error is occured"
            response = {"valid": False, "message": message,
                        "code": 400, "data": {}}
            return Response(response, status=400)


class SongDetailsView(generics.ListAPIView):
    permission_classes = (IsAuthenticated,)
    authentication_classes = (JWTAuthentication,)
    queryset = Songs.objects.filter(isDeleted=False, status=True).all()
    serializer_class = SongsSerializerList
    filter_backends = (filters.DjangoFilterBackend,)
    filterset_class = SongsFilter

    def list(self, request, *args, **kwargs):
        try:
            queryset = self.filter_queryset(self.get_queryset())
            if queryset:
                songObj = queryset.first()
                songObj.listen_count = songObj.listen_count + 1
                songObj.save()
                serializer = self.get_serializer(
                    instance=queryset.first(), context={"request": self.request}
                )
                response = {
                    "valid": True,
                    "code": 200,
                    "message": "Data fetch successfully",
                    "data": {"songs": serializer.data},
                }
                return Response(response, status=200)
            else:
                message = "Song Does Not Exist"
                response = {"valid": False, "message": message,
                            "code": 400, "data": {}}
                return Response(response, status=400)
        except:
            message = "Some error is occured"
            response = {"valid": False, "message": message,
                        "code": 400, "data": {}}
            return Response(response, status=400)


class MostPopularSongsView(generics.ListAPIView):
    permission_classes = (IsAuthenticated,)
    authentication_classes = (JWTAuthentication,)
    queryset = Songs.objects.filter(isDeleted=False, status=True).all()
    serializer_class = SongsSerializerList

    def list(self, request, *args, **kwargs):
        try:
            queryset = self.get_queryset()
            queryset = queryset.filter(
                isDeleted=False, status=True,
                created__lte=datetime.today(),
                created__gt=datetime.today() - timedelta(days=60),
            ).order_by("listen_count")[:10]
            serializer = self.get_serializer(
                instance=queryset, many=True, context={"request": self.request}
            )
            response = {
                "valid": True,
                "code": 200,
                "message": "Data fetch successfully",
                "data": {"songs": serializer.data},
            }
            return Response(response, status=200)
        except:
            message = "Some error is occured"
            response = {"valid": False, "message": message,
                        "code": 400, "data": {}}
            return Response(response, status=400)


class UpdateTodayspecialVideoCountView(generics.GenericAPIView):
    permission_classes = (IsAuthenticated,)
    authentication_classes = (JWTAuthentication,)
    serializer_class = UpdateTodayspecialVideoSerializer

    def put(self, request):
        try:
            today_special_id = request.data.get("id")
            try:
                todaySpecialObj = TodaySpecial.objects.get(id=today_special_id)
            except:
                todaySpecialObj = None
            if todaySpecialObj:
                video = todaySpecialObj.video
                video.listen_count = video.listen_count + 1
                video.save()
                response = {
                    "valid": True,
                    "code": 200,
                    "message": "Data updated successfully",
                    "data": {},
                }
                return Response(response, status=200)
            else:
                message = "Does Not Exist"
                response = {"valid": False, "message": message,
                            "code": 400, "data": {}}
                return Response(response, status=400)
        except Exception as e:
            message = "Some error is occured"
            response = {"valid": False, "message": message,
                        "code": 400, "data": {}}
            return Response(response, status=400)


class StaticPageView(generics.ListAPIView):
    permission_classes = (IsAuthenticated,)
    authentication_classes = (JWTAuthentication,)
    queryset = Page.objects.all()
    serializer_class = PageSerializerList
    filter_backends = (filters.DjangoFilterBackend,)
    filterset_class = PageFilter

    def get_serializer_context(self):
        context = super(StaticPageView, self).get_serializer_context()
        context.update({"request": self.request})
        return context

    def list(self, request, *args, **kwargs):
        try:
            url = ""
            page_role = request.GET.get("pagerole")
            if page_role == "privacy_policy":
                url = "http://35.202.49.65/App-Privacy-Policy-SOUL%20JOURNEY.pdf"
            if page_role == "terms_and_condition":
                url = "http://35.202.49.65/TERMS%20&%20CONDITIONS%20SOUL%20JOURNEY.pdf"
            queryset = self.filter_queryset(self.get_queryset())
            serializer = self.get_serializer(instance=queryset, many=True)
            response = {
                "valid": True,
                "code": 200,
                "message": "Data fetch successfully",
                "data": {"page": serializer.data, "page_url": url},
            }
            return Response(response, status=200)
        except:
            message = "Some error is occured"
            response = {"valid": False, "message": message,
                        "code": 400, "data": {}}
            return Response(response, status=400)


@permission_classes((permissions.AllowAny,))
class ContactUsView(generics.CreateAPIView):

    serializer_class = ContactUsSerializer

    def create(self, request, *args, **kwarg):
        try:
            data = request.data
            serializer = self.get_serializer(data=request.data)
            if serializer.is_valid():
                serializer.save()
                response = {
                    "code": 200,
                    "data": {},
                    "message": "Query submitted successfully ",
                }
                return Response(response, status=200)
            else:
                try:
                    message = list(serializer.errors.values())[0][0]
                except:
                    message = "some error is occured please try again"
                response = {"code": 400, "message": message}

                return Response(response, status=400)
        except:
            message = "Some error is occured"
            response = {"valid": False, "message": message,
                        "code": 400, "data": {}}
            return Response(response, status=400)


@permission_classes((permissions.AllowAny,))
class LoginPageVideoView(generics.ListAPIView):
    queryset = LoginPageVideo.objects.all()
    serializer_class = LoginPageVideoSerializerList

    def get_serializer_context(self):
        context = super(LoginPageVideoView, self).get_serializer_context()
        context.update({"request": self.request})
        return context

    def list(self, request, *args, **kwargs):
        try:
            queryset = self.get_queryset()
            queryset = queryset.last()
            serializer = self.get_serializer(instance=queryset)
            response = {
                "valid": True,
                "code": 200,
                "message": "Data fetch successfully",
                "data": {"video": serializer.data},
            }
            return Response(response, status=200)
        except:
            message = "Some error is occured"
            response = {"valid": False, "message": message,
                        "code": 400, "data": {}}
            return Response(response, status=400)


class AddAlarmNotificationView(generics.GenericAPIView):
    serializer_class = AddAlarmNotificationSerializer
    queryset = User.objects.all()

    def post(self, request):
        try:
            token = jwt.decode(request.headers['Authorization'].split(' ')[
                               1], settings.SECRET_KEY)
            timezone = request.data.get("timezone")
            if not timezone:
                response = {"code": 400,
                            "message": "Timezone is required", "data": {}}
                return Response(response, status=400)
            serializer = self.get_serializer(data=request.data)
            if serializer.is_valid():
                user_id = serializer.data.get("user")
                device_id = token['device_id']
                device_type = token['device_type']
                set_time = serializer.data.get("set_time")
                end_time = serializer.data.get(
                    "end_time"
                )  # "2021-08-05 07:52:00 UTC+0000" for '%Y-%m-%d %H:%M:%S %Z%z'
                print(set_time,end_time,'Set 1st')
                try:
                    set_time = datetime.strptime(set_time, "%Y-%m-%d %H:%M:%S")
                    end_time = datetime.strptime(end_time, "%Y-%m-%d %H:%M:%S")

                    user_timezone = pytz.timezone(timezone)
                    new_timezone = pytz.timezone("Asia/Kolkata")
                    new_time = user_timezone.localize(set_time).astimezone(new_timezone) 
                    new_end_time = user_timezone.localize(end_time).astimezone(new_timezone)
                    #set_time = user_timezone.localize(set_time, is_dst=None)
                    # set_time = set_time.astimezone(user_timezone)
                    #end_time = user_timezone.localize(end_time, is_dst=None)
                    # print(set_time,'<><><<<><><><>')
                    # end_time = end_time.astimezone(user_timezone)
                    # print(set_time,end_time,'Set 2nd')
                    # format = "%Y-%m-%d %H:%M:%S"
                    # set_time = set_time.strftime(format)
                    # set_time = datetime.strptime(set_time, "%Y-%m-%d %H:%M:%S")
                    # end_time = end_time.strftime(format)
                    # end_time = datetime.strptime(end_time, "%Y-%m-%d %H:%M:%S")
                    #print(set_time,end_time,datetime.now() + timedelta(minutes=29),'Set &')
                except Exception as e:
                    response = {"code": 400, "message": str(e), "data": {}}
                    return Response(response, status=400)
                try:
                    user = User.objects.get(id=user_id)
                except:
                    user = None
                    response = {
                        "code": 400,
                        "message": "User Does Not Exist",
                        "data": {},
                    }
                    return Response(response, status=400)

                FCMDevice.objects.filter(
                    user=user, device_id=device_id, type=device_type).update(active=True)

                notification_data = {
                    "user_id": str(user.id),
                    "songsPlayList": serializer.data.get("category_id"),
                }
                set_time = new_time.replace(tzinfo=None)
                end_time = new_end_time.replace(tzinfo=None)
                #print(set_time,end_time,'<><><><><><>')
                notify_user_alarm_set_time(
                    notification_data, schedule=set_time, queue=user.id)
                notify_user_alarm_end_time(
                    notification_data, schedule=end_time, queue=user.id)
                response = {
                    "code": 200,
                    "data": {},
                    "message": "success",
                }
                return Response(response, status=200)
            else:
                try:
                    message = list(serializer.errors.values())[0][0]
                except Exception as e:
                    print("AddAlarmNotificationView :: error :: ", e)
                    print("AddAlarmNotificationView :: traceback :: ", traceback.format_exc())
                    message = "some error is occured please try again"
                response = {"code": 400, "message": message, "data": {}}

                return Response(response, status=400)
        except Exception as e:
            print("AddAlarmNotificationView :: >>>>>>>>>> :: error", e)
            print("AddAlarmNotificationView :: >>>>>>>>>> :: traceback", traceback.format_exc())
            response = {"code": 400, "message": "Something wrong", "data": {}}
            return Response(response, status=400)


class AddDurationNotificationView(generics.GenericAPIView):
    serializer_class = AddDurationNotificationSerializer
    queryset = User.objects.all()

    def post(self, request):
        try:
            token = jwt.decode(request.headers['Authorization'].split(' ')[
                               1], settings.SECRET_KEY)
            serializer = self.get_serializer(data=request.data)
            timezone = request.data.get("timezone")
            if not timezone:
                response = {"code": 400,
                            "message": "Timezone is required", "data": {}}
                return Response(response, status=400)
            if serializer.is_valid():
                device_id = token['device_id']
                device_type = token['device_type']
                user_id = serializer.data.get("user")
                current_time = serializer.data.get("current_time")
                duration = serializer.data.get("duration_in_min")
                try:
                    current_time = datetime.strptime(
                        current_time, "%Y-%m-%d %H:%M:%S")
                    user_timezone = pytz.timezone(timezone)
                    current_time = user_timezone.localize(
                        current_time, is_dst=None)
                    current_time = current_time.astimezone(
                        pytz.timezone("Asia/Kolkata")
                    )
                    format = "%Y-%m-%d %H:%M:%S"
                    current_time = current_time.strftime(format)
                    current_time = datetime.strptime(
                        current_time, "%Y-%m-%d %H:%M:%S")
                    duration = timedelta(minutes=int(duration))
                    current_time = current_time + duration
                except Exception as e:
                    response = {"code": 400, "message": str(e), "data": {}}
                    return Response(response, status=400)
                try:
                    user = User.objects.get(id=user_id)
                except:
                    user = None
                    response = {
                        "code": 400,
                        "message": "User Does Not Exist",
                        "data": {},
                    }
                    return Response(response, status=400)
                
                FCMDevice.objects.filter(
                    user=user, device_id=device_id, type=device_type).update(active=True)

                notification_data = {
                    "user_id": str(user.id),
                    "songsPlayList": serializer.data.get("category_id"),
                }
                notify_user_alarm_duration_time(
                    notification_data, schedule=current_time
                )
                response = {
                    "code": 200,
                    "data": {},
                    "message": "success",
                }
                return Response(response, status=200)
            else:
                try:
                    message = list(serializer.errors.values())[0][0]
                except:
                    message = "some error is occured please try again"
                response = {"code": 400, "message": message, "data": {}}

                return Response(response, status=400)
        except Exception as e:
            print(">>>>>>>>>>", e)
            response = {"code": 400, "message": "Something wrong", "data": {}}
            return Response(response, status=400)


class UserNotificationListView(generics.ListAPIView):
    queryset = UserNotification.objects.all()
    serializer_class = UserNotificationSerializerList

    def list(self, request, *args, **kwargs):
        try:
            user = request.user
            queryset = self.get_queryset()
            queryset = queryset.filter(user=user)
            serializer = self.get_serializer(instance=queryset, many=True)
            response = {
                "valid": True,
                "code": 200,
                "message": "Data fetch successfully",
                "data": {"notifications": serializer.data},
            }
            return Response(response, status=200)
        except Exception as e:
            message = "Some error is occured"
            response = {"valid": False, "message": message,
                        "code": 400, "data": {}}
            return Response(response, status=400)


class SubscriptionReceiptValidation(generics.CreateAPIView):
    permission_classes = (IsAuthenticated,)
    authentication_classes = (JWTAuthentication,)
    queryset = UserSubscription.objects.all()
    serializer_class = SubscriptionReceiptSerializer

    def create(self, request, *args, **kwarg):

        try:
            data = request.data
            data["user"] = str(request.user.id)

            serializer = self.get_serializer(data=data)
            if serializer.is_valid():
                serializer.save()
                response = {
                    "valid": True,
                    "code": 200,
                    "message": "Subcription done successfully.",
                    "data": {},
                }
                self.request.user.cancel_subscription_applied = False
                self.request.user.save()
                return Response(response, status=201)
            else:
                try:
                    message = list(serializer.errors.values())[0][0]
                    print(serializer.errors)
                except Exception as e:
                    message = f"some error is occured please try again: {e}"
                response = {"valid": False, "code": 400,
                            "message": message, "data": {}}

                return Response(response, status=400)
        except Exception as e:
            message = f"Some error is occured: {e}"
            response = {"valid": False, "message": message,
                        "code": 400, "data": {}}
            return Response(response, status=400)


class CheckSubscriptionReceipt(APIView):
    permission_classes = (IsAuthenticated,)
    authentication_classes = (JWTAuthentication,)

    def patch(self, request, format=None):

        response = {
            "valid": True,
            "code": 200,
            "message": "",
            "data": {},
        }
        cancel_subscription_applied = self.request.user.cancel_subscription_applied
        try:
            subcription = UserSubscription.objects.get(user=self.request.user)
            response["data"] = {"subcription": subcription.check_subcription()}
        except UserSubscription.DoesNotExist:
            response["data"] = {
                "subcription": False,
                "cancel_subscription_applied": cancel_subscription_applied,
            }
            return Response(response, status=200)
        response["data"][
            "cancel_subscription_applied"
        ] = cancel_subscription_applied

        if cancel_subscription_applied:
            response[
                "message"] = f"Your have already applied for cancel subscription request, Your existing subscription will be valid till date {subcription.expirationDate}"
        return Response(response, status=200)


class cancelSubscription(APIView):
    permission_classes = (IsAuthenticated,)
    authentication_classes = (JWTAuthentication,)

    def patch(self, request, format=None):
        response = {
            "valid": True,
            "code": 200,
            "message": "",
            "data": {},
        }
        try:
            subcription = UserSubscription.objects.get(user=self.request.user)
            if subcription.status == False:
                response = {
                    "valid": False,
                    "message": "Already unsubscribed",
                    "code": 404,
                    "data": {},
                }
                return Response(response, status=404)
            else:
                response[
                    "message"
                ] = f"You subscription has been cancelled and wouldn't be charged onward but you can use your subscription untill {subcription.expirationDate}."
                response["data"] = {
                    "subcription_cancel": subcription.cancelsubcription()
                }
        except UserSubscription.DoesNotExist:
            message = "No subcription found."
            response = {"valid": False, "message": message,
                        "code": 404, "data": {}}
            return Response(response, status=404)
        self.request.user.cancel_subscription_applied = True
        self.request.user.save()
        response["data"][
            "cancel_subscription_applied"
        ] = self.request.user.cancel_subscription_applied
        return Response(response, status=200)


class accountDelete(APIView):
    permission_classes = (IsAuthenticated,)
    authentication_classes = (JWTAuthentication,)

    def delete(self, request, format=None):
        User.objects.filter(id=self.request.user.id).delete()
        response = {
            "valid": True,
            "message": "Your account has been deleted successfully.",
            "code": 200,
            "data": {},
        }
        return Response(response, status=200)

class downloadedSongsView(generics.ListCreateAPIView):
    permission_classes = (IsAuthenticated,)
    authentication_classes = (JWTAuthentication,)
    serializer_class = DownloadedSongsSerializer
    queryset = DownloadedSongs.objects.all()
    filter_backends = (filters.DjangoFilterBackend,)
    filterset_fields = ['device_id', 'device_type']

    def get_queryset(self):
        return self.queryset.filter(user=self.request.user)

    def song_count(self):
        return self.queryset.filter(user=self.request.user).aggregate(Count('songs'))['songs__count']

    def list(self, request, *args, **kwargs):
        try:
            queryset = self.filter_queryset(self.get_queryset())
            serializer = self.get_serializer(instance=queryset, many=True)
            response = {
                "valid": True,
                "code": 200,
                "message": "Data fetch successfully",
                "data": {"download_songs": serializer.data, "count": self.song_count()},
            }
            return Response(response, status=200)
        except Exception as e:
            message = "Some error is occured"
            response = {"valid": False, "message": message,
                        "code": 400, "data": {}}
            return Response(response, status=400)
    

    def create(self, request, *args, **kwargs):
        try:
            songs_id = request.data.pop('songs_id', None)
            if songs_id:
                request.data['songs'] = [songs_id]
            serializer = self.get_serializer(data={'user':self.request.user.id, **request.data})
            if self.song_count() > 2 :
                subcription = UserSubscription.objects.filter(user=self.request.user)
                if subcription.exists():
                    if subcription[0].status == False:
                        response = {"valid": False, "code": 400, "message": "You need to subscribe to download more songs."}
                        return Response(response, status=400)
                else:
                    response = {"valid": False, "code": 400, "message": "You need to subscribe to download more songs."}
                    return Response(response, status=400)
            if serializer.is_valid():
                serializer.save()
                response = {"valid": True, "code": 200, "data": serializer.data, "message": "Song added to download list successfully."}
            else:
                response = {"valid": False, "code": 400, "data": serializer.errors, "message": "Something went wrong."}
            return Response(response, status=200)
        except Exception as e:
            message = f"Some error is occured: {e}"
            response = {"message": message, "code": 400, "data": {}}
            return Response(response, status=400)

class deletedownloadedSongs(generics.DestroyAPIView):
    permission_classes = (IsAuthenticated,)
    authentication_classes = (JWTAuthentication,)
    serializer_class = DownloadedSongsSerializer
    queryset = DownloadedSongs.objects.all()

    def get_queryset(self):
        return self.queryset.filter(user=self.request.user)
    
    def destroy(self, request, *args, **kwargs):
        obj = DownloadedSongs.objects.get(**kwargs)
        songs = request.GET.getlist('song')
        for song in Songs.objects.filter(id__in=songs):
            obj.songs.remove(song)
        return Response(status=204)
    