var express = require('express');
var iap = require("iap");
var serviceAccount_new = require("./../google-service-account-credentials.json");
var router = express.Router();
var path = require('path');
/* GET users listing. */

var pathc = path.resolve(process.cwd(), '../.env')
require('dotenv').config({ path: pathc })

router.get('/', function(req, res, next) {
  try{
      let platform = req.body.platform;
        if(!platform || platform === undefined) {
          return res.status(403).json({ "success": false, "error_code": 403, "message": "Platform not found" });
        }
        let payment = {}
        if(platform == "apple"){
            payment = {
            receipt: req.body.receipt, 
            productId: req.body.productId,
            packageName: req.body.packageName,
            secret:process.env.APPLE_SECRET_KEY,
            subscription: true, 
            keyObject: serviceAccount_new
        };
      }
      if(platform == "google"){
        payment = {
          receipt: req.body.receipt, 
          productId: req.body.productId,
          packageName: req.body.packageName,
          subscription: true, 
          keyObject: serviceAccount_new
      }
    }

    if(!payment.receipt || payment.receipt == undefined){
      return res.send({status: false, statusCode: 402, response: "Please provide receipt Id"})
    }

    if(!payment.productId || payment.productId == undefined){
      return res.send({status: false, statusCode: 402, response: "Please provide productId"})
    }
  
    if(!payment.packageName || payment.packageName == undefined){
      return res.send({status: false, statusCode: 402, response: "Please provide packageName"})
    }
        iap.verifyPayment(platform, payment, function (error, response) {
            if (error) {
              return res.status(403).send({
                "success": false,
                "error_code": 403,
                "message": error
            });
            }
            const currDate = new Date();
            const timestamp = Date.parse(currDate);
            if (response.expirationDate > timestamp) {
              return res.send({
                "success": true,
                "data": response
              });
            } else {
              return res.status(403).json({ "success": false, "error_code": 403, "message": "Subscription Expired" });
            }
        });
  }catch(error){
    return res.status(400).json({ "success": false, "error_code": 40, "message": error});
  }
});


router.post('/cancelSubscription', function(req, res, next) {
    try{

      let platform = req.body.platform;
      if(!platform || platform === undefined) {
        return res.status(403).json({ "success": false, "error_code": 403, "message": "Platform not found" });
      }
      let paymentData = {}
      if(platform == "apple"){
       paymentData = {
        receipt: req.body.receipt, 
        productId: req.body.productId,
        packageName: req.body.packageName,
        secret:process.env.APPLE_SECRET_KEY,
        subscription: true, 
        keyObject: serviceAccount_new
    };
  }
  if(platform == "google"){
    paymentData = {
      receipt: req.body.receipt, 
        receipt: req.body.receipt, 
      receipt: req.body.receipt, 
      productId: req.body.productId,
      packageName: req.body.packageName,
      subscription: true, 
      keyObject: serviceAccount_new
  };
  }
    if(!paymentData.receipt || paymentData.receipt == undefined){
      return res.send({status: false, statusCode: 402, response: "Please provide receipt Id"})
    }

    if(!paymentData.productId || paymentData.productId == undefined){
      return res.send({status: false, statusCode: 402, response: "Please provide productId"})
    }

    if(!paymentData.packageName || paymentData.packageName == undefined){
      return res.send({status: false, statusCode: 402, response: "Please provide packageName"})
    }
    iap.cancelSubscription(platform, paymentData, function (error, response){
      
        if (error) {
          return res.status(403).send({
            "success": false,
            "error_code": 403,
            "message": error
        });
        }
        return res.status(200).json({ "success": true, "code": 200 , "message": "Subscription Cancelled" });
    });

    }catch(error){
      console.log(error)
      return res.status(400).json({status: false, statusCode: 400, message: "Error in cancel subscription"})
    }
});

module.exports = router;
